# Copyright (C) 2011-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""REST for users."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'AUser',
    'AllUsers',
    ]


from flufl.password import lookup, make_secret, generate
from restish import http, resource
from uuid import UUID
from zope.component import getUtility

from mailman.config import config
from mailman.interfaces.address import ExistingAddressError
from mailman.interfaces.usermanager import IUserManager
from mailman.rest.addresses import UserAddresses
from mailman.rest.helpers import CollectionMixin, etag, no_content, path_to
from mailman.rest.preferences import Preferences
from mailman.rest.validator import Validator



class _UserBase(resource.Resource, CollectionMixin):
    """Shared base class for user representations."""

    def _resource_as_dict(self, user):
        """See `CollectionMixin`."""
        # The canonical URL for a user is their unique user id, although we
        # can always look up a user based on any registered and validated
        # email address associated with their account.  The user id is a UUID,
        # but we serialize its integer equivalent.
        user_id = user.user_id.int
        resource = dict(
            user_id=user_id,
            created_on=user.created_on,
            self_link=path_to('users/{0}'.format(user_id)),
            )
        # Add the password attribute, only if the user has a password.  Same
        # with the real name.  These could be None or the empty string.
        if user.password:
            resource['password'] = user.password
        if user.display_name:
            resource['display_name'] = user.display_name
        return resource

    def _get_collection(self, request):
        """See `CollectionMixin`."""
        return list(getUtility(IUserManager).users)



class AllUsers(_UserBase):
    """The users."""

    @resource.GET()
    def collection(self, request):
        """/users"""
        resource = self._make_collection(request)
        return http.ok([], etag(resource))

    @resource.POST()
    def create(self, request):
        """Create a new user."""
        try:
            validator = Validator(email=unicode,
                                  display_name=unicode,
                                  password=unicode,
                                  _optional=('display_name', 'password'))
            arguments = validator(request)
        except ValueError as error:
            return http.bad_request([], str(error))
        # We can't pass the 'password' argument to the user creation method,
        # so strip that out (if it exists), then create the user, adding the
        # password after the fact if successful.
        password = arguments.pop('password', None)
        try:
            user = getUtility(IUserManager).create_user(**arguments)
        except ExistingAddressError as error:
            return http.bad_request([], b'Address already exists {0}'.format(
                error.email))
        if password is None:
            # This will have to be reset since it cannot be retrieved.
            password = generate(int(config.passwords.password_length))
        scheme = lookup(config.passwords.password_scheme.upper())
        user.password = make_secret(password, scheme)
        location = path_to('users/{0}'.format(user.user_id.int))
        return http.created(location, [], None)



class AUser(_UserBase):
    """A user."""

    def __init__(self, user_identifier):
        """Get a user by various type of identifiers.

        :param user_identifier: The identifier used to retrieve the user.  The
            identifier may either be an integer user-id, or an email address
            controlled by the user.  The type of identifier is auto-detected
            by looking for an `@` symbol, in which case it's taken as an email
            address, otherwise it's assumed to be an integer.
        :type user_identifier: string
        """
        user_manager = getUtility(IUserManager)
        if '@' in user_identifier:
            self._user = user_manager.get_user(user_identifier)
        else:
            # The identifier is the string representation of an integer that
            # must be converted to a UUID.
            try:
                user_id = UUID(int=int(user_identifier))
            except ValueError:
                self._user = None
            else:
                self._user = user_manager.get_user_by_id(user_id)

    @resource.GET()
    def user(self, request):
        """Return a single user end-point."""
        if self._user is None:
            return http.not_found()
        return http.ok([], self._resource_as_json(self._user))

    @resource.child()
    def addresses(self, request, segments):
        """/users/<uid>/addresses"""
        return UserAddresses(self._user)

    @resource.DELETE()
    def delete_user(self, request):
        """Delete the named user."""
        if self._user is None:
            return http.not_found()
        getUtility(IUserManager).delete_user(self._user)
        return no_content()

    @resource.child()
    def preferences(self, request, segments):
        """/addresses/<email>/preferences"""
        if len(segments) != 0:
            return http.bad_request()
        if self._user is None:
            return http.not_found()
        child = Preferences(
            self._user.preferences,
            'users/{0}'.format(self._user.user_id.int))
        return child, []



# Copyright (C) 2007-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Implementation of the IUserRegistrar interface."""

from __future__ import absolute_import, print_function, unicode_literals

__metaclass__ = type
__all__ = [
    'Registrar',
    ]


import logging

from zope.component import getUtility
from zope.interface import implements

from mailman.app.notifications import send_welcome_message
from mailman.core.i18n import _
from mailman.email.message import UserNotification
from mailman.interfaces.address import IEmailValidator
from mailman.interfaces.listmanager import IListManager
from mailman.interfaces.member import DeliveryMode, MemberRole
from mailman.interfaces.pending import IPendable, IPendings
from mailman.interfaces.registrar import IRegistrar
from mailman.interfaces.templates import ITemplateLoader
from mailman.interfaces.usermanager import IUserManager
from mailman.utilities.datetime import now


log = logging.getLogger('mailman.error')



class PendableRegistration(dict):
    implements(IPendable)
    PEND_KEY = 'registration'



class Registrar:
    """Handle registrations and confirmations for subscriptions."""

    implements(IRegistrar)

    def register(self, mlist, email, display_name=None, delivery_mode=None):
        """See `IUserRegistrar`."""
        if delivery_mode is None:
            delivery_mode = DeliveryMode.regular
        # First, do validation on the email address.  If the address is
        # invalid, it will raise an exception, otherwise it just returns.
        getUtility(IEmailValidator).validate(email)
        # Create a pendable for the registration.
        pendable = PendableRegistration(
            type=PendableRegistration.PEND_KEY,
            email=email,
            display_name=display_name,
            delivery_mode=delivery_mode.name)
        pendable['list_name'] = mlist.fqdn_listname
        token = getUtility(IPendings).add(pendable)
        # There are three ways for a user to confirm their subscription.  They
        # can reply to the original message and let the VERP'd return address
        # encode the token, they can reply to the robot and keep the token in
        # the Subject header, or they can click on the URL in the body of the
        # message and confirm through the web.
        subject = 'confirm ' + token
        confirm_address = mlist.confirm_address(token)
        # For i18n interpolation.
        confirm_url = mlist.domain.confirm_url(token)
        email_address = email
        domain_name = mlist.domain.mail_host
        contact_address = mlist.domain.contact_address
        # Send a verification email to the address.
        template = getUtility(ITemplateLoader).get(
            'mailman:///{0}/{1}/confirm.txt'.format(
                mlist.fqdn_listname,
                mlist.preferred_language.code))
        text = _(template)
        msg = UserNotification(email, confirm_address, subject, text)
        msg.send(mlist)
        return token

    def confirm(self, token):
        """See `IUserRegistrar`."""
        # For convenience
        pendable = getUtility(IPendings).confirm(token)
        if pendable is None:
            return False
        missing = object()
        email = pendable.get('email', missing)
        display_name = pendable.get('display_name', missing)
        list_name = pendable.get('list_name', missing)
        pended_delivery_mode = pendable.get('delivery_mode', 'regular')
        try:
            delivery_mode = DeliveryMode[pended_delivery_mode]
        except ValueError:
            log.error('Invalid pended delivery_mode for {0}: {1}',
                      email, pended_delivery_mode)
            delivery_mode = DeliveryMode.regular
        if pendable.get('type') != PendableRegistration.PEND_KEY:
            # It seems like it would be very difficult to accurately guess
            # tokens, or brute force an attack on the SHA1 hash, so we'll just
            # throw the pendable away in that case.  It's possible we'll need
            # to repend the event or adjust the API to handle this case
            # better, but for now, the simpler the better.
            return False
        # We are going to end up with an IAddress for the verified address
        # and an IUser linked to this IAddress.  See if any of these objects
        # currently exist in our database.
        user_manager = getUtility(IUserManager)
        address = (user_manager.get_address(email)
                   if email is not missing else None)
        user = (user_manager.get_user(email)
                if email is not missing else None)
        # If there is neither an address nor a user matching the confirmed
        # record, then create the user, which will in turn create the address
        # and link the two together
        if address is None:
            assert user is None, 'How did we get a user but not an address?'
            user = user_manager.create_user(email, display_name)
            # Because the database changes haven't been flushed, we can't use
            # IUserManager.get_address() to find the IAddress just created
            # under the hood.  Instead, iterate through the IUser's addresses,
            # of which really there should be only one.
            for address in user.addresses:
                if address.email == email:
                    break
            else:
                raise AssertionError('Could not find expected IAddress')
        elif user is None:
            user = user_manager.create_user()
            user.display_name = display_name
            user.link(address)
        else:
            # The IAddress and linked IUser already exist, so all we need to
            # do is verify the address.
            pass
        address.verified_on = now()
        # If this registration is tied to a mailing list, subscribe the person
        # to the list right now, and possibly send a welcome message.
        list_name = pendable.get('list_name')
        if list_name is not None:
            mlist = getUtility(IListManager).get(list_name)
            if mlist:
                member = mlist.subscribe(address, MemberRole.member)
                member.preferences.delivery_mode = delivery_mode
                if mlist.send_welcome_message:
                    send_welcome_message(mlist,
                                         address.email,
                                         mlist.preferred_language,
                                         delivery_mode)
        return True

    def discard(self, token):
        # Throw the record away.
        getUtility(IPendings).confirm(token)



# Copyright (C) 2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Test the message API."""

from __future__ import absolute_import, print_function, unicode_literals

__metaclass__ = type
__all__ = [
    'TestMessage',
    ]


import unittest

from mailman.app.lifecycle import create_list
from mailman.email.message import UserNotification
from mailman.testing.helpers import get_queue_messages
from mailman.testing.layers import ConfigLayer



class TestMessage(unittest.TestCase):
    """Test the message API."""

    layer = ConfigLayer

    def setUp(self):
        self._mlist = create_list('test@example.com')
        self._msg = UserNotification(
            'aperson@example.com',
            'test@example.com',
            'Something you need to know',
            'I needed to tell you this.')

    def test_one_precedence_header(self):
        # Ensure that when the original message already has a Precedence:
        # header, UserNotification.send(..., add_precedence=True, ...) does
        # not add a second header.
        self.assertEqual(self._msg['precedence'], None)
        self._msg['Precedence'] = 'omg wtf bbq'
        self._msg.send(self._mlist)
        messages = get_queue_messages('virgin')
        self.assertEqual(len(messages), 1)
        self.assertEqual(messages[0].msg.get_all('precedence'), 
                         ['omg wtf bbq'])



# Copyright (C) 2009-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Information about this Mailman instance."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'Info'
    ]


import sys

from zope.interface import implements

from mailman.config import config
from mailman.core.i18n import _
from mailman.interfaces.command import ICLISubCommand
from mailman.rest.helpers import path_to
from mailman.version import MAILMAN_VERSION_FULL



class Info:
    """Information about this Mailman instance."""

    implements(ICLISubCommand)

    name = 'info'

    def add(self, parser, command_parser):
        """See `ICLISubCommand`."""
        command_parser.add_argument(
            '-o', '--output',
            action='store', help=_("""\
            File to send the output to.  If not given, standard output is
            used."""))
        command_parser.add_argument(
            '-v', '--verbose',
            action='store_true', help=_("""\
            A more verbose output including the file system paths that Mailman
            is using."""))

    def process(self, args):
        """See `ICLISubCommand`."""
        if args.output is None:
            output = sys.stdout
        else:
            # We don't need to close output because that will happen
            # automatically when the script exits.
            output = open(args.output, 'w')
        print >> output, MAILMAN_VERSION_FULL
        print >> output, 'Python', sys.version
        print >> output, 'config file:', config.filename
        print >> output, 'db url:', config.db.url
        print >> output, 'REST root url:', path_to('/')
        print >> output, 'REST credentials: {0}:{1}'.format(
            config.webservice.admin_user, config.webservice.admin_pass)
        if args.verbose:
            print >> output, 'File system paths:'
            longest = 0
            paths = {}
            for attribute in dir(config):
                if attribute.endswith('_DIR') or attribute.endswith('_FILE'):
                    paths[attribute] = getattr(config, attribute)
                longest = max(longest, len(attribute))
            for attribute in sorted(paths):
                print '    {0:{2}} = {1}'.format(attribute, paths[attribute],
                                                 longest)



# Copyright (C) 2009-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Module stuff."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'ContentFilter'
    ]


from storm.locals import Int, Reference, Unicode
from zope.interface import implements

from mailman.database.model import Model
from mailman.database.types import Enum
from mailman.interfaces.mime import IContentFilter, FilterType



class ContentFilter(Model):
    """A single filter criteria."""
    implements(IContentFilter)

    id = Int(primary=True)

    mailing_list_id = Int()
    mailing_list = Reference(mailing_list_id, 'MailingList.id')

    filter_type = Enum(FilterType)
    filter_pattern = Unicode()

    def __init__(self, mailing_list, filter_pattern, filter_type):
        self.mailing_list = mailing_list
        self.filter_pattern = filter_pattern
        self.filter_type = filter_type



# Copyright (C) 2011-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Manager of email address bans."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'IBan',
    'IBanManager',
    ]


from zope.interface import Attribute, Interface



class IBan(Interface):
    """A specific ban.

    In general, this interface isn't publicly useful.
    """

    email = Attribute('The banned email address, or pattern.')

    mailing_list = Attribute(
        """The fqdn name of the mailing list the ban applies to.

        Use None if this is a global ban.
        """)



class IBanManager(Interface):
    """The global manager of email address bans."""

    def ban(email, mailing_list=None):
        """Ban an email address from subscribing to a mailing list.

        When an email address is banned, it will not be allowed to subscribe
        to a the named mailing list.  This does not affect any email address
        already subscribed to the mailing list.  With the default arguments,
        an email address can be banned globally from subscribing to any
        mailing list on the system.

        It is also possible to add a 'ban pattern' whereby all email addresses
        matching a Python regular expression can be banned.  This is
        accomplished by using a `^` as the first character in `email`.

        When an email address is already banned for the given mailing list (or
        globally), then this method does nothing.  However, it is possible to
        extend a ban for a specific mailing list into a global ban; both bans
        would be in place and they can be removed individually.

        :param email: The text email address being banned or, if the string
            starts with a caret (^), the email address pattern to ban.
        :type email: str
        :param mailing_list: The fqdn name of the mailing list to which the
            ban applies.  If None, then the ban is global.
        :type mailing_list: string
        """

    def unban(email, mailing_list=None):
        """Remove an email address ban.

        This removes a specific or global email address ban, which would have
        been added with the `ban()` method.  If a ban is lifted which did not
        exist, this method does nothing.

        :param email: The text email address being unbanned or, if the string
            starts with a caret (^), the email address pattern to unban.
        :type email: str
        :param mailing_list: The fqdn name of the mailing list to which the
            unban applies.  If None, then the unban is global.
        :type mailing_list: string
        """

    def is_banned(email, mailing_list=None):
        """Check whether a specific email address is banned.

        `email` must be a text email address; it cannot be a pattern.  The
        given email address is checked against all registered bans, both
        specific and regular expression, both for the named mailing list (if
        given), and globally.

        :param email: The text email address being checked.
        :type email: str
        :param mailing_list: The fqdn name of the mailing list being checked.
            Note that if not None, both specific and global bans will be
            checked.  If None, then only global bans will be checked.
        :type mailing_list: string
        :return: A flag indicating whether the given email address is banned
            or not.
        :rtype: bool
        """



# Copyright (C) 2009-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Membership interface for REST."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'ISubscriptionService',
    ]


from zope.interface import Interface

from mailman.interfaces.errors import MailmanError
from mailman.interfaces.member import DeliveryMode, MemberRole



class MissingUserError(MailmanError):
    """A an invalid user id was given."""

    def __init__(self, user_id):
        super(MissingUserError, self).__init__()
        self.user_id = user_id

    def __str__(self):
        return self.user_id



class ISubscriptionService(Interface):
    """General Subscription services."""

    def get_members():
        """Return a sequence of all members of all mailing lists.

        The members are sorted first by fully-qualified mailing list name,
        then by subscribed email address, then by role.  Because the user may
        be a member of the list under multiple roles (e.g. as an owner and as
        a digest member), the member can appear multiple times in this list.
        Roles are sorted by: owner, moderator, member.

        :return: The list of all members.
        :rtype: list of `IMember`
        """

    def get_member(member_id):
        """Return a member record matching the member id.

        :param member_id: A member id.
        :type member_id: int
        :return: The matching member, or None if no matching member is found.
        :rtype: `IMember`
        """

    def find_members(subscriber=None, fqdn_listname=None, role=None):
        """Search for and return a specific member.

        The members are sorted first by fully-qualified mailing list name,
        then by subscribed email address, then by role.  Because the user may
        be a member of the list under multiple roles (e.g. as an owner and as
        a digest member), the member can appear multiple times in this list.

        :param subscriber: The email address or user id of the user getting
            subscribed.
        :type subscriber: string or int
        :param fqdn_listname: The posting address of the mailing list to
            search for the subscriber's memberships on.
        :type fqdn_listname: string
        :param role: The member role.
        :type role: `MemberRole`
        :return: The list of all memberships, which may be empty.
        :rtype: list of `IMember`
        """

    def __iter__():
        """See `get_members()`."""

    def join(fqdn_listname, subscriber, display_name=None,
             delivery_mode=DeliveryMode.regular, 
             role=MemberRole.member):
        """Subscribe to a mailing list.

        A user for the address is created if it is not yet known to Mailman,
        however newly registered addresses will not yet be validated.  No
        confirmation message will be sent to the address, and the approval of
        the subscription request is still dependent on the policy of the
        mailing list.

        :param fqdn_listname: The posting address of the mailing list to
            subscribe the user to.
        :type fqdn_listname: string
        :param subscriber: The email address or user id of the user getting
            subscribed.
        :type subscriber: string or int
        :param display_name: The name of the user.  This is only used if a new
            user is created, and it defaults to the local part of the email
            address if not given.
        :type display_name: string
        :param delivery_mode: The delivery mode for this subscription.  This
            can be one of the enum values of `DeliveryMode`.  If not given,
            regular delivery is assumed.
        :type delivery_mode: string
        :param role: The membership role for this subscription.
        :type role: `MemberRole`
        :return: The just created member.
        :rtype: `IMember`
        :raises AlreadySubscribedError: if the user is already subscribed to
            the mailing list.
        :raises InvalidEmailAddressError: if the email address is not valid.
        :raises MembershipIsBannedError: if the membership is not allowed.
        :raises MissingUserError: when a bogus user id is given.
        :raises NoSuchListError: if the named mailing list does not exist.
        :raises ValueError: when `delivery_mode` is invalid.
        """

    def leave(fqdn_listname, email):
        """Unsubscribe from a mailing list.

        :param fqdn_listname: The posting address of the mailing list to
            unsubscribe the user from.
        :type fqdn_listname: string
        :param email: The email address of the user getting unsubscribed.
        :type email: string
        :raises InvalidEmailAddressError: if the email address is not valid.
        :raises NoSuchListError: if the named mailing list does not exist.
        :raises NotAMemberError: if the given address is not a member of the
            mailing list.
        """



# Copyright (C) 2011-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""REST address tests."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    ]


import unittest

from urllib2 import HTTPError

from mailman.app.lifecycle import create_list
from mailman.config import config
from mailman.testing.helpers import call_api
from mailman.testing.layers import RESTLayer



class TestAddresses(unittest.TestCase):
    layer = RESTLayer

    def setUp(self):
        self._mlist = create_list('test@example.com')
        config.db.commit()

    def test_membership_of_missing_address(self):
        # Try to get the memberships of a missing address.
        try:
            # For Python 2.6.
            call_api('http://localhost:9001/3.0/addresses/'
                     'nobody@example.com/memberships')
        except HTTPError as exc:
            self.assertEqual(exc.code, 404)
        else:
            raise AssertionError('Expected HTTPError')



# Copyright (C) 2008-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Domains."""

from __future__ import unicode_literals

__metaclass__ = type
__all__ = [
    'Domain',
    'DomainManager',
    ]


from urlparse import urljoin, urlparse
from storm.locals import Int, Unicode
from zope.event import notify
from zope.interface import implements

from mailman.config import config
from mailman.database.model import Model
from mailman.interfaces.domain import (
    BadDomainSpecificationError, DomainCreatedEvent, DomainCreatingEvent,
    DomainDeletedEvent, DomainDeletingEvent, IDomain, IDomainManager)
from mailman.model.mailinglist import MailingList



class Domain(Model):
    """Domains."""

    implements(IDomain)

    id = Int(primary=True)

    mail_host = Unicode()
    base_url = Unicode()
    description = Unicode()
    contact_address = Unicode()

    def __init__(self, mail_host,
                 description=None,
                 base_url=None,
                 contact_address=None):
        """Create and register a domain.

        :param mail_host: The host name for the email interface.
        :type mail_host: string
        :param description: An optional description of the domain.
        :type description: string
        :param base_url: The optional base url for the domain, including
            scheme.  If not given, it will be constructed from the
            `mail_host` using the http protocol.
        :type base_url: string
        :param contact_address: The email address to contact a human for this
            domain.  If not given, postmaster@`mail_host` will be used.
        :type contact_address: string
        """
        self.mail_host = mail_host
        self.base_url = (base_url
                         if base_url is not None
                         else 'http://' + mail_host)
        self.description = description
        self.contact_address = (contact_address
                                if contact_address is not None
                                else 'postmaster@' + mail_host)

    @property
    def url_host(self):
        """See `IDomain`."""
        return urlparse(self.base_url).netloc

    @property
    def scheme(self):
        """See `IDomain`."""
        return urlparse(self.base_url).scheme

    @property
    def mailing_lists(self):
        """See `IDomain`."""
        mailing_lists = config.db.store.find(
            MailingList,
            MailingList.mail_host == self.mail_host)
        for mlist in mailing_lists:
            yield mlist

    def confirm_url(self, token=''):
        """See `IDomain`."""
        return urljoin(self.base_url, 'confirm/' + token)

    def __repr__(self):
        """repr(a_domain)"""
        if self.description is None:
            return ('<Domain {0.mail_host}, base_url: {0.base_url}, '
                    'contact_address: {0.contact_address}>').format(self)
        else:
            return ('<Domain {0.mail_host}, {0.description}, '
                    'base_url: {0.base_url}, '
                    'contact_address: {0.contact_address}>').format(self)



class DomainManager:
    """Domain manager."""

    implements(IDomainManager)

    def add(self, mail_host,
            description=None,
            base_url=None,
            contact_address=None):
        """See `IDomainManager`."""
        # Be sure the mail_host is not already registered.  This is probably
        # a constraint that should (also) be maintained in the database.
        if self.get(mail_host) is not None:
            raise BadDomainSpecificationError(
                'Duplicate email host: %s' % mail_host)
        notify(DomainCreatingEvent(mail_host))
        domain = Domain(mail_host, description, base_url, contact_address)
        config.db.store.add(domain)
        notify(DomainCreatedEvent(domain))
        return domain

    def remove(self, mail_host):
        domain = self[mail_host]
        notify(DomainDeletingEvent(domain))
        config.db.store.remove(domain)
        notify(DomainDeletedEvent(mail_host))
        return domain

    def get(self, mail_host, default=None):
        """See `IDomainManager`."""
        domains = config.db.store.find(Domain, mail_host=mail_host)
        if domains.count() < 1:
            return default
        assert domains.count() == 1, (
            'Too many matching domains: %s' % mail_host)
        return domains.one()

    def __getitem__(self, mail_host):
        """See `IDomainManager`."""
        missing = object()
        domain = self.get(mail_host, missing)
        if domain is missing:
            raise KeyError(mail_host)
        return domain

    def __len__(self):
        return config.db.store.find(Domain).count()

    def __iter__(self):
        """See `IDomainManager`."""
        for domain in config.db.store.find(Domain):
            yield domain

    def __contains__(self, mail_host):
        """See `IDomainManager`."""
        return config.db.store.find(Domain, mail_host=mail_host).count() > 0



# Copyright (C) 1998-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Incoming runner.

This runner's sole purpose in life is to decide the disposition of the
message.  It can either be accepted for delivery, rejected (i.e. bounced),
held for moderator approval, or discarded.

When accepted, the message is forwarded on to the `prep queue` where it is
prepared for delivery.  Rejections, discards, and holds are processed
immediately.
"""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'IncomingRunner',
    ]


from zope.component import getUtility

from mailman.config import config
from mailman.core.chains import process
from mailman.core.runner import Runner
from mailman.interfaces.address import ExistingAddressError
from mailman.interfaces.usermanager import IUserManager



class IncomingRunner(Runner):
    """The incoming runner."""

    def _dispose(self, mlist, msg, msgdata):
        """See `IRunner`."""
        if msgdata.get('envsender') is None:
            msgdata['envsender'] = mlist.no_reply_address
        # Ensure that the email addresses of the message's senders are known
        # to Mailman.  This will be used in nonmember posting dispositions.
        user_manager = getUtility(IUserManager)
        for sender in msg.senders:
            try:
                user_manager.create_address(sender)
            except ExistingAddressError:
                pass
        config.db.commit()
        # Process the message through the mailing list's start chain.
        start_chain = (mlist.owner_chain
                       if msgdata.get('to_owner', False)
                       else mlist.posting_chain)
        process(mlist, msg, msgdata, start_chain)
        # Do not keep this message queued.
        return False



# Copyright (C) 2010-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""REST for domains."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'ADomain',
    'AllDomains',
    ]


from restish import http, resource
from zope.component import getUtility

from mailman.interfaces.domain import (
    BadDomainSpecificationError, IDomainManager)
from mailman.rest.helpers import CollectionMixin, etag, no_content, path_to
from mailman.rest.lists import ListsForDomain
from mailman.rest.validator import Validator



class _DomainBase(resource.Resource, CollectionMixin):
    """Shared base class for domain representations."""

    def _resource_as_dict(self, domain):
        """See `CollectionMixin`."""
        return dict(
            base_url=domain.base_url,
            contact_address=domain.contact_address,
            description=domain.description,
            mail_host=domain.mail_host,
            self_link=path_to('domains/{0}'.format(domain.mail_host)),
            url_host=domain.url_host,
            )

    def _get_collection(self, request):
        """See `CollectionMixin`."""
        return list(getUtility(IDomainManager))


class ADomain(_DomainBase):
    """A domain."""

    def __init__(self, domain):
        self._domain = domain

    @resource.GET()
    def domain(self, request):
        """Return a single domain end-point."""
        domain = getUtility(IDomainManager).get(self._domain)
        if domain is None:
            return http.not_found()
        return http.ok([], self._resource_as_json(domain))

    @resource.DELETE()
    def delete(self, request):
        """Delete the domain."""
        try:
            getUtility(IDomainManager).remove(self._domain)
        except KeyError:
            # The domain does not exist.
            return http.not_found()
        return no_content()

    @resource.child()
    def lists(self, request, segments):
        """/domains/<domain>/lists"""
        if len(segments) == 0:
            domain = getUtility(IDomainManager).get(self._domain)
            if domain is None:
                return http.not_found()
            return ListsForDomain(domain)
        else:
            return http.bad_request()


class AllDomains(_DomainBase):
    """The domains."""

    @resource.POST()
    def create(self, request):
        """Create a new domain."""
        domain_manager = getUtility(IDomainManager)
        try:
            validator = Validator(mail_host=unicode,
                                  description=unicode,
                                  base_url=unicode,
                                  contact_address=unicode,
                                  _optional=('description', 'base_url',
                                             'contact_address'))
            domain = domain_manager.add(**validator(request))
        except BadDomainSpecificationError:
            return http.bad_request([], b'Domain exists')
        except ValueError as error:
            return http.bad_request([], str(error))
        location = path_to('domains/{0}'.format(domain.mail_host))
        # Include no extra headers or body.
        return http.created(location, [], None)

    @resource.GET()
    def collection(self, request):
        """/domains"""
        resource = self._make_collection(request)
        return http.ok([], etag(resource))



# Copyright (C) 2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Test the core modification pipelines."""

from __future__ import absolute_import, print_function, unicode_literals

__metaclass__ = type
__all__ = [
    'TestOwnerPipeline',
    'TestPostingPipeline',
    ]


import unittest

from zope.component import getUtility
from zope.interface import implements

from mailman.app.lifecycle import create_list
from mailman.config import config
from mailman.core.errors import DiscardMessage, RejectMessage
from mailman.core.pipelines import process
from mailman.interfaces.handler import IHandler
from mailman.interfaces.member import MemberRole
from mailman.interfaces.pipeline import IPipeline
from mailman.interfaces.usermanager import IUserManager
from mailman.testing.helpers import (
    LogFileMark,
    get_queue_messages,
    reset_the_world,
    specialized_message_from_string as mfs)
from mailman.testing.layers import ConfigLayer



class DiscardingHandler:
    implements(IHandler)
    name = 'discarding'

    def process(self, mlist, msg, msgdata):
        raise DiscardMessage('by test handler')


class RejectHandler:
    implements(IHandler)
    name = 'rejecting'

    def process(self, mlist, msg, msgdata):
        raise RejectMessage('by test handler')


class DiscardingPipeline:
    implements(IPipeline)
    name = 'test-discarding'
    description = 'Discarding test pipeline'

    def __iter__(self):
        yield DiscardingHandler()


class RejectingPipeline:
    implements(IPipeline)
    name = 'test-rejecting'
    description = 'Rejectinging test pipeline'

    def __iter__(self):
        yield RejectHandler()



class TestPostingPipeline(unittest.TestCase):
    """Test various aspects of the built-in postings pipeline."""

    layer = ConfigLayer

    def setUp(self):
        self._mlist = create_list('test@example.com')
        config.pipelines['test-discarding'] = DiscardingPipeline()
        config.pipelines['test-rejecting'] = RejectingPipeline()
        self._msg = mfs("""\
From: Anne Person <anne@example.org>
To: test@example.com
Subject: a test
Message-ID: <ant>

testing
""")

    def tearDown(self):
        reset_the_world()
        del config.pipelines['test-discarding']
        del config.pipelines['test-rejecting']

    def test_rfc2369_headers(self):
        # Ensure that RFC 2369 List-* headers are added.
        msgdata = {}
        process(self._mlist, self._msg, msgdata,
                pipeline_name='default-posting-pipeline')
        self.assertEqual(self._msg['list-id'], '<test.example.com>')
        self.assertEqual(self._msg['list-post'], '<mailto:test@example.com>')

    def test_discarding_pipeline(self):
        # If a handler in the pipeline raises DiscardMessage, the message will
        # be thrown away, but with a log message.
        mark = LogFileMark('mailman.vette')
        process(self._mlist, self._msg, {}, 'test-discarding')
        line = mark.readline()[:-1]
        self.assertTrue(line.endswith(
            '<ant> discarded by "test-discarding" pipeline handler '
            '"discarding": by test handler'))

    def test_rejecting_pipeline(self):
        # If a handler in the pipeline raises DiscardMessage, the message will
        # be thrown away, but with a log message.
        mark = LogFileMark('mailman.vette')
        process(self._mlist, self._msg, {}, 'test-rejecting')
        line = mark.readline()[:-1]
        self.assertTrue(line.endswith(
            '<ant> rejected by "test-rejecting" pipeline handler '
            '"rejecting": by test handler'))
        # In the rejection case, the original message will also be in the
        # virgin queue.
        messages = get_queue_messages('virgin')
        self.assertEqual(len(messages), 1)
        self.assertEqual(str(messages[0].msg['subject']), 'a test')



class TestOwnerPipeline(unittest.TestCase):
    """Test various aspects of the built-in owner pipeline."""

    layer = ConfigLayer

    def setUp(self):
        self._mlist = create_list('test@example.com')
        user_manager = getUtility(IUserManager)
        anne = user_manager.create_address('anne@example.com')
        bart = user_manager.create_address('bart@example.com')
        self._mlist.subscribe(anne, MemberRole.owner)
        self._mlist.subscribe(bart, MemberRole.moderator)
        self._msg = mfs("""\
From: Anne Person <anne@example.org>
To: test-owner@example.com

""")

    def test_calculate_recipients(self):
        # Recipients are the administrators of the mailing list.
        msgdata = dict(listname='test@example.com',
                       to_owner=True)
        process(self._mlist, self._msg, msgdata,
                pipeline_name='default-owner-pipeline')
        self.assertEqual(msgdata['recipients'], set(('anne@example.com',
                                                     'bart@example.com')))

    def test_to_outgoing(self):
        # The message, with the calculated recipients, gets put in the
        # outgoing queue.
        process(self._mlist, self._msg, {},
                pipeline_name='default-owner-pipeline')
        messages = get_queue_messages('out', sort_on='to')
        self.assertEqual(len(messages), 1)
        self.assertEqual(messages[0].msgdata['recipients'], 
                         set(('anne@example.com', 'bart@example.com')))



# Copyright (C) 2011-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""PostgreSQL database support."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'PostgreSQLDatabase',
    ]


from operator import attrgetter

from mailman.database.base import StormBaseDatabase



class PostgreSQLDatabase(StormBaseDatabase):
    """Database class for PostgreSQL."""

    TAG = 'postgres'

    def _database_exists(self, store):
        """See `BaseDatabase`."""
        table_query = ('SELECT table_name FROM information_schema.tables '
                       "WHERE table_schema = 'public'")
        table_names = set(item[0] for item in
                          store.execute(table_query))
        return 'version' in table_names

    def _post_reset(self, store):
        """PostgreSQL-specific test suite cleanup.

        Reset the <tablename>_id_seq.last_value so that primary key ids
        restart from zero for new tests.
        """
        super(PostgreSQLDatabase, self)._post_reset(store)
        from mailman.database.model import ModelMeta
        classes = sorted(ModelMeta._class_registry,
                         key=attrgetter('__storm_table__'))
        # Recipe adapted from
        # http://stackoverflow.com/questions/544791/
        # django-postgresql-how-to-reset-primary-key
        for model_class in classes:
            store.execute("""\
                SELECT setval('"{0}_id_seq"', coalesce(max("id"), 1),
                              max("id") IS NOT null)
                       FROM "{0}";
                """.format(model_class.__storm_table__))



# Copyright (C) 1998-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

import re
import sys
import optparse

from mailman import errors
from mailman import MailList
from mailman.configuration import config
from mailman.core.i18n import _
from mailman.version import MAILMAN_VERSION


AS_MEMBER   = 0x01
AS_OWNER    = 0x02



def parseargs():
    parser = optparse.OptionParser(version=MAILMAN_VERSION,
                                   usage=_("""\
%prog [options] regex [regex ...]

Find all lists that a member's address is on.

The interaction between -l and -x (see below) is as follows.  If any -l option
is given then only the named list will be included in the search.  If any -x
option is given but no -l option is given, then all lists will be search
except those specifically excluded.

Regular expression syntax uses the Python 're' module.  Complete
specifications are at:

http://www.python.org/doc/current/lib/module-re.html

Address matches are case-insensitive, but case-preserved addresses are
displayed."""))
    parser.add_option('-l', '--listname',
                      type='string', default=[], action='append',
                      dest='listnames',
                      help=_('Include only the named list in the search'))
    parser.add_option('-x', '--exclude',
                      type='string', default=[], action='append',
                      dest='excludes',
                      help=_('Exclude the named list from the search'))
    parser.add_option('-w', '--owners',
                      default=False, action='store_true',
                      help=_('Search list owners as well as members'))
    parser.add_option('-C', '--config',
                      help=_('Alternative configuration file to use'))
    opts, args = parser.parse_args()
    if not args:
        parser.print_help()
        print >> sys.stderr, _('Search regular expression required')
        sys.exit(1)
    return parser, opts, args



def main():
    parser, opts, args = parseargs()
    config.load(opts.config)

    listnames = opts.listnames or config.list_manager.names
    includes = set(listname.lower() for listname in listnames)
    excludes = set(listname.lower() for listname in opts.excludes)
    listnames = includes - excludes

    if not listnames:
        print _('No lists to search')
        return

    cres = []
    for r in args:
        cres.append(re.compile(r, re.IGNORECASE))
    # dictionary of {address, (listname, ownerp)}
    matches = {}
    for listname in listnames:
        try:
            mlist = MailList.MailList(listname, lock=False)
        except errors.MMListError:
            print _('No such list: $listname')
            continue
        if opts.owners:
            owners = mlist.owner
        else:
            owners = []
        for cre in cres:
            for member in mlist.getMembers():
                if cre.search(member):
                    addr = mlist.getMemberCPAddress(member)
                    entries = matches.get(addr, {})
                    aswhat = entries.get(listname, 0)
                    aswhat |=  AS_MEMBER
                    entries[listname] = aswhat
                    matches[addr] = entries
            for owner in owners:
                if cre.search(owner):
                    entries = matches.get(owner, {})
                    aswhat = entries.get(listname, 0)
                    aswhat |= AS_OWNER
                    entries[listname] = aswhat
                    matches[owner] = entries
    addrs = matches.keys()
    addrs.sort()
    for k in addrs:
        hits = matches[k]
        lists = hits.keys()
        print k, _('found in:')
        for name in lists:
            aswhat = hits[name]
            if aswhat & AS_MEMBER:
                print '    ', name
            if aswhat & AS_OWNER:
                print '    ', name, _('(as owner)')



if __name__ == '__main__':
    main()



# Copyright (C) 2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Test the various pending requests interfaces."""

from __future__ import absolute_import, print_function, unicode_literals

__metaclass__ = type
__all__ = [
    ]


import unittest

from mailman.app.lifecycle import create_list
from mailman.app.moderator import hold_message
from mailman.interfaces.requests import IListRequests, RequestType
from mailman.testing.helpers import specialized_message_from_string as mfs
from mailman.testing.layers import ConfigLayer



class TestRequests(unittest.TestCase):
    layer = ConfigLayer

    def setUp(self):
        self._mlist = create_list('ant@example.com')
        self._msg = mfs("""\
From: anne@example.com
To: ant@example.com
Subject: Something
Message-ID: <alpha>

Something else.
""")

    def test_get_request_with_type(self):
        # get_request() takes an optional request type.
        request_id = hold_message(self._mlist, self._msg)
        requests_db = IListRequests(self._mlist)
        # Submit a request with a non-matching type.  This should return None
        # as if there were no matches.
        response = requests_db.get_request(
            request_id, RequestType.subscription)
        self.assertEqual(response, None)
        # Submit the same request with a matching type.
        key, data = requests_db.get_request(
            request_id, RequestType.held_message)
        self.assertEqual(key, '<alpha>')
        # It should also succeed with no optional request type given.
        key, data = requests_db.get_request(request_id)
        self.assertEqual(key, '<alpha>')



# Copyright (C) 2011-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Ban manager."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'BanManager',
    ]


import re

from storm.locals import Int, Unicode
from zope.interface import implements

from mailman.config import config
from mailman.database.model import Model
from mailman.interfaces.bans import IBan, IBanManager



class Ban(Model):
    implements(IBan)

    id = Int(primary=True)
    email = Unicode()
    mailing_list = Unicode()

    def __init__(self, email, mailing_list):
        super(Ban, self).__init__()
        self.email = email
        self.mailing_list = mailing_list



class BanManager:
    implements(IBanManager)

    def ban(self, email, mailing_list=None):
        """See `IBanManager`."""
        bans = config.db.store.find(
            Ban, email=email, mailing_list=mailing_list)
        if bans.count() == 0:
            ban = Ban(email, mailing_list)
            config.db.store.add(ban)

    def unban(self, email, mailing_list=None):
        """See `IBanManager`."""
        ban = config.db.store.find(
            Ban, email=email, mailing_list=mailing_list).one()
        if ban is not None:
            config.db.store.remove(ban)

    def is_banned(self, email, mailing_list=None):
        """See `IBanManager`."""
        # A specific mailing list ban is being checked, however the email
        # address could be banned specifically, or globally.
        if mailing_list is not None:
            # Try specific bans first.
            bans = config.db.store.find(
                Ban, email=email, mailing_list=mailing_list)
            if bans.count() > 0:
                return True
            # Try global bans next.
            bans = config.db.store.find(Ban, email=email, mailing_list=None)
            if bans.count() > 0:
                return True
            # Now try specific mailing list bans, but with a pattern.
            bans = config.db.store.find(Ban, mailing_list=mailing_list)
            for ban in bans:
                if (ban.email.startswith('^') and
                    re.match(ban.email, email, re.IGNORECASE) is not None):
                    return True
            # And now try global pattern bans.
            bans = config.db.store.find(Ban, mailing_list=None)
            for ban in bans:
                if (ban.email.startswith('^') and
                    re.match(ban.email, email, re.IGNORECASE) is not None):
                    return True
        else:
            # The client is asking for global bans.  Look up bans on the
            # specific email address first.
            bans = config.db.store.find(
                Ban, email=email, mailing_list=None)
            if bans.count() > 0:
                return True
            # And now look for global pattern bans.
            bans = config.db.store.find(Ban, mailing_list=None)
            for ban in bans:
                if (ban.email.startswith('^') and
                    re.match(ban.email, email, re.IGNORECASE) is not None):
                    return True
        return False



# Copyright (C) 2009-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""String utilities."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'expand',
    'oneline',
    'uncanonstr',
    'websafe',
    'wrap',
    ]


import cgi
import logging

from email.errors import HeaderParseError
from email.header import decode_header, make_header
from string import Template, whitespace
from textwrap import TextWrapper, dedent
from zope.component import getUtility

from mailman.interfaces.languages import ILanguageManager


EMPTYSTRING = ''
NL = '\n'

log = logging.getLogger('mailman.error')



def expand(template, substitutions, template_class=Template):
    """Expand string template with substitutions.

    :param template: A PEP 292 $-string template.
    :type template: string
    :param substitutions: The substitutions dictionary.
    :type substitutions: dict
    :param template_class: The template class to use.
    :type template_class: class
    :return: The substituted string.
    :rtype: string
    """
    # Python 2.6 requires ** dictionaries to have str, not unicode keys, so
    # convert as necessary.  Note that string.Template uses **.  For our
    # purposes, keys should always be ascii.  Values though can be anything.
    cooked = substitutions.__class__()
    for key in substitutions:
        if isinstance(key, unicode):
            key = key.encode('ascii')
        cooked[key] = substitutions[key]
    try:
        return template_class(template).safe_substitute(cooked)
    except (TypeError, ValueError):
        # The template is really screwed up.
        log.exception('broken template: %s', template)



def oneline(s, cset='us-ascii', in_unicode=False):
    """Decode a header string in one line and convert into specified charset.

    :param s: The header string
    :type s: string
    :param cset: The character set (encoding) to use.
    :type cset: string
    :param in_unicode: Flag specifying whether to return the converted string
        as a unicode (True) or an 8-bit string (False, the default).
    :type in_unicode: bool
    :return: The decoded header string.  If an error occurs while converting
        the input string, return the string undecoded, as an 8-bit string.
    :rtype: string
    """
    try:
        h = make_header(decode_header(s))
        ustr = h.__unicode__()
        line = EMPTYSTRING.join(ustr.splitlines())
        if in_unicode:
            return line
        else:
            return line.encode(cset, 'replace')
    except (LookupError, UnicodeError, ValueError, HeaderParseError):
        # possibly charset problem. return with undecoded string in one line.
        return EMPTYSTRING.join(s.splitlines())



def websafe(s):
    return cgi.escape(s, quote=True)



# The opposite of canonstr() -- sorta.  I.e. it attempts to encode s in the
# charset of the given language, which is the character set that the page will
# be rendered in, and failing that, replaces non-ASCII characters with their
# html references.  It always returns a byte string.
def uncanonstr(s, lang=None):
    if s is None:
        s = u''
    if lang is None:
        charset = 'us-ascii'
    else:
        charset = getUtility(ILanguageManager)[lang].charset
    # See if the string contains characters only in the desired character
    # set.  If so, return it unchanged, except for coercing it to a byte
    # string.
    try:
        if isinstance(s, unicode):
            return s.encode(charset)
        else:
            unicode(s, charset)
            return s
    except UnicodeError:
        # Nope, it contains funny characters, so html-ref it
        a = []
        for c in s:
            o = ord(c)
            if o > 127:
                a.append('&#%3d;' % o)
            else:
                a.append(c)
        # Join characters together and coerce to byte string
        return str(EMPTYSTRING.join(a))



def wrap(text, column=70, honor_leading_ws=True):
    """Wrap and fill the text to the specified column.

    The input text is wrapped and filled as done by the standard library
    textwrap module.  The differences here being that this function is capable
    of filling multiple paragraphs (as defined by text separated by blank
    lines).  Also, when `honor_leading_ws` is True (the default), paragraphs
    that being with whitespace are not wrapped.  This is the algorithm that
    the Python FAQ wizard used.
    """
    # First, split the original text into paragraph, keeping all blank lines
    # between them.
    paragraphs = []
    paragraph = []
    last_indented = False
    for line in text.splitlines(True):
        is_indented = (len(line) > 0 and line[0] in whitespace)
        if line == NL:
            if len(paragraph) > 0:
                paragraphs.append(EMPTYSTRING.join(paragraph))
            paragraphs.append(line)
            last_indented = False
            paragraph = []
        elif last_indented != is_indented:
            # The indentation level changed.  We treat this as a paragraph
            # break but no blank line will be issued between paragraphs.
            if len(paragraph) > 0:
                paragraphs.append(EMPTYSTRING.join(paragraph))
            # The next paragraph starts with this line.
            paragraph = [line]
            last_indented = is_indented
        else:
            # This line does not constitute a paragraph break.
            paragraph.append(line)
    # We've consumed all the lines in the original text.  Transfer the last
    # paragraph we were collecting to the full set of paragraphs.
    paragraphs.append(EMPTYSTRING.join(paragraph))
    # Now iterate through all paragraphs, wrapping as necessary.
    wrapped_paragraphs = []
    # The dedented wrapper.
    wrapper = TextWrapper(width=column,
                          fix_sentence_endings=True)
    # The indented wrapper.  For this one, we'll clobber initial_indent and
    # subsequent_indent as needed per indented chunk of text.
    iwrapper = TextWrapper(width=column,
                           fix_sentence_endings=True,
                           )
    add_paragraph_break = False
    for paragraph in paragraphs:
        if add_paragraph_break:
            wrapped_paragraphs.append(NL)
            add_paragraph_break = False
        paragraph_text = EMPTYSTRING.join(paragraph)
        # Just copy the blank lines to the final set of paragraphs.
        if len(paragraph) == 0 or paragraph == NL:
            wrapped_paragraphs.append(NL)
        # Choose the wrapper based on whether the paragraph is indented or
        # not.  Also, do not wrap indented paragraphs if honor_leading_ws is
        # set.
        elif paragraph[0] in whitespace:
            if honor_leading_ws:
                # Leave the indented paragraph verbatim.
                wrapped_paragraphs.append(paragraph_text)
            else:
                # The paragraph should be wrapped, but it must first be
                # dedented.  The leading whitespace on the first line of the
                # original text will be used as the indentation for all lines
                # in the wrapped text.
                for i, ch in enumerate(paragraph_text):
                    if ch not in whitespace:
                        break
                leading_ws = paragraph[:i]
                iwrapper.initial_indent=leading_ws
                iwrapper.subsequent_indent=leading_ws
                paragraph_text = dedent(paragraph_text)
                wrapped_paragraphs.append(iwrapper.fill(paragraph_text))
                add_paragraph_break = True
        else:
            # Fill this paragraph.  fill() consumes the trailing newline.
            wrapped_paragraphs.append(wrapper.fill(paragraph_text))
            add_paragraph_break = True
    return EMPTYSTRING.join(wrapped_paragraphs)



# Copyright (C) 2007-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Interface describing a user registration service.

This is a higher level interface to user registration, address confirmation,
etc. than the IUserManager.  The latter does no validation, syntax checking,
or confirmation, while this interface does.
"""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'IRegistrar',
    ]


from zope.interface import Interface



class IRegistrar(Interface):
    """Interface for registering and verifying email addresses and users.

    This is a higher level interface to user registration, email address
    confirmation, etc. than the IUserManager.  The latter does no validation,
    syntax checking, or confirmation, while this interface does.
    """

    def register(mlist, email, display_name=None, delivery_mode=None):
        """Register the email address, requesting verification.

        No `IAddress` or `IUser` is created during this step, but after
        successful confirmation, it is guaranteed that an `IAddress` with a
        linked `IUser` will exist.  When a verified `IAddress` matching
        `email` already exists, this method will do nothing, except link a new
        `IUser` to the `IAddress` if one is not yet associated with the
        email address.

        In all cases, the email address is sanity checked for validity first.

        :param mlist: The mailing list that is the focus of this registration.
        :type mlist: `IMailingList`
        :param email: The email address to register.
        :type email: str
        :param display_name: The optional display name of the user.
        :type display_name: str
        :param delivery_mode: The optional delivery mode for this
            registration.  If not given, regular delivery is used.
        :type delivery_mode: `DeliveryMode`
        :return: The confirmation token string.
        :rtype: str
        :raises InvalidEmailAddressError: if the address is not allowed.
        """

    def confirm(token):
        """Confirm the pending registration matched to the given `token`.

        Confirmation ensures that the IAddress exists and is linked to an
        IUser, with the latter being created and linked if necessary.

        :param token: A token matching a pending event with a type of
            'registration'.
        :return: Boolean indicating whether the confirmation succeeded or
            not.  It may fail if the token is no longer in the database, or if
            the token did not match a registration event.
        """

    def discard(token):
        """Discard the pending registration matched to the given `token`.

        The event record is discarded and the IAddress is not verified.  No
        IUser is created.

        :param token: A token matching a pending event with a type of
            'registration'.
        """



# Copyright (C) 2010-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Basic WSGI Application object for REST server."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'make_application',
    'make_server',
    ]


import logging

from restish.app import RestishApp
from wsgiref.simple_server import WSGIRequestHandler
from wsgiref.simple_server import make_server as wsgi_server

from mailman.config import config
from mailman.rest.root import Root


log = logging.getLogger('mailman.http')



class AdminWebServiceWSGIRequestHandler(WSGIRequestHandler):
    """Handler class which just logs output to the right place."""

    def log_message(self, format, *args):
        """See `BaseHTTPRequestHandler`."""
        log.info('%s - - %s', self.address_string(), format % args)


class AdminWebServiceApplication(RestishApp):
    """Connect the restish WSGI application to Mailman's database."""

    def __call__(self, environ, start_response):
        """See `RestishApp`."""
        try:
            response = super(AdminWebServiceApplication, self).__call__(
                environ, start_response)
        except:
            config.db.abort()
            raise
        else:
            config.db.commit()
            return response



def make_application():
    """Create the WSGI application.

    Use this if you want to integrate Mailman's REST server with your own WSGI
    server.
    """
    return AdminWebServiceApplication(Root())


def make_server():
    """Create the Mailman REST server.

    Use this if you just want to run Mailman's wsgiref-based REST server.
    """
    host = config.webservice.hostname
    port = int(config.webservice.port)
    server = wsgi_server(
        host, port, make_application(),
        handler_class=AdminWebServiceWSGIRequestHandler)
    return server



# Copyright (C) 2011-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""REST list tests."""

from __future__ import absolute_import, print_function, unicode_literals

__metaclass__ = type
__all__ = [
    'TestLists',
    'TestListsMissing',
    ]


import unittest

from urllib2 import HTTPError
from zope.component import getUtility

from mailman.app.lifecycle import create_list
from mailman.config import config
from mailman.interfaces.usermanager import IUserManager
from mailman.testing.helpers import call_api
from mailman.testing.layers import RESTLayer



class TestListsMissing(unittest.TestCase):
    """Test expected failures."""

    layer = RESTLayer

    def test_missing_list_roster_member_404(self):
        # /lists/<missing>/roster/member gives 404
        try:
            # For Python 2.6.
            call_api('http://localhost:9001/3.0/lists/missing@example.com'
                     '/roster/member')
        except HTTPError as exc:
            self.assertEqual(exc.code, 404)
        else:
            raise AssertionError('Expected HTTPError')

    def test_missing_list_roster_owner_404(self):
        # /lists/<missing>/roster/owner gives 404
        try:
            # For Python 2.6.
            call_api('http://localhost:9001/3.0/lists/missing@example.com'
                     '/roster/owner')
        except HTTPError as exc:
            self.assertEqual(exc.code, 404)
        else:
            raise AssertionError('Expected HTTPError')

    def test_missing_list_roster_moderator_404(self):
        # /lists/<missing>/roster/member gives 404
        try:
            # For Python 2.6.
            call_api('http://localhost:9001/3.0/lists/missing@example.com'
                     '/roster/moderator')
        except HTTPError as exc:
            self.assertEqual(exc.code, 404)
        else:
            raise AssertionError('Expected HTTPError')

    def test_missing_list_configuration_404(self):
        # /lists/<missing>/config gives 404
        try:
            # For Python 2.6.
            call_api(
                'http://localhost:9001/3.0/lists/missing@example.com/config')
        except HTTPError as exc:
            self.assertEqual(exc.code, 404)
        else:
            raise AssertionError('Expected HTTPError')



class TestLists(unittest.TestCase):
    """Test various aspects of mailing list resources."""

    layer = RESTLayer

    def setUp(self):
        self._mlist = create_list('test@example.com')
        config.db.commit()
        self._usermanager = getUtility(IUserManager)

    def test_member_count_with_no_members(self):
        # The list initially has 0 members.
        resource, response = call_api(
            'http://localhost:9001/3.0/lists/test@example.com')
        self.assertEqual(response.status, 200)
        self.assertEqual(resource['member_count'], 0)

    def test_member_count_with_one_member(self):
        # Add a member to a list and check that the resource reflects this.
        anne = self._usermanager.create_address('anne@example.com')
        self._mlist.subscribe(anne)
        config.db.commit()
        resource, response = call_api(
            'http://localhost:9001/3.0/lists/test@example.com')
        self.assertEqual(response.status, 200)
        self.assertEqual(resource['member_count'], 1)

    def test_member_count_with_two_members(self):
        # Add two members to a list and check that the resource reflects this.
        anne = self._usermanager.create_address('anne@example.com')
        self._mlist.subscribe(anne)
        bart = self._usermanager.create_address('bar@example.com')
        self._mlist.subscribe(bart)
        config.db.commit()
        resource, response = call_api(
            'http://localhost:9001/3.0/lists/test@example.com')
        self.assertEqual(response.status, 200)
        self.assertEqual(resource['member_count'], 2)



# Copyright (C) 2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Test styles."""

from __future__ import absolute_import, print_function, unicode_literals

__metaclass__ = type
__all__ = [
    'TestStyle',
    ]


import unittest

from zope.component import getUtility
from zope.interface import implements
from zope.interface.exceptions import DoesNotImplement

from mailman.interfaces.styles import (
    DuplicateStyleError, IStyle, IStyleManager)
from mailman.testing.layers import ConfigLayer



class DummyStyle:
    implements(IStyle)

    name = 'dummy'
    priority = 1

    def apply(self, mlist):
        pass

    def match(self, mlist, styles):
        styles.append(self)



class TestStyle(unittest.TestCase):
    """Test styles."""

    layer = ConfigLayer

    def setUp(self):
        self.manager = getUtility(IStyleManager)

    def test_register_style_again(self):
        # Registering a style with the same name as a previous style raises an
        # exception.
        self.manager.register(DummyStyle())
        try:
            self.manager.register(DummyStyle())
        except DuplicateStyleError:
            pass
        else:
            raise AssertionError('DuplicateStyleError exception expected')

    def test_register_a_non_style(self):
        # You can't register something that doesn't implement the IStyle
        # interface.
        try:
            self.manager.register(object())
        except DoesNotImplement:
            pass
        else:
            raise AssertionError('DoesNotImplement exception expected')

    def test_unregister_a_non_registered_style(self):
        # You cannot unregister a style that hasn't yet been registered.
        try:
            self.manager.unregister(DummyStyle())
        except KeyError:
            pass
        else:
            raise AssertionError('KeyError expected')



# Copyright (C) 2001-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Creation/deletion hooks for the MTAs that do things automatically.

Exim one example of an MTA that Just Works.
"""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'NullMTA',
    ]


from zope.interface import implements
from mailman.interfaces.mta import IMailTransportAgentLifecycle



class NullMTA:
    """Null MTA that just satisfies the interface."""

    implements(IMailTransportAgentLifecycle)

    def create(self, mlist):
        """See `IMailTransportAgentLifecycle`."""
        pass

    def delete(self, mlist):
        """See `IMailTransportAgentLifecycle`."""
        pass

    def regenerate(self, output=None):
        """See `IMailTransportAgentLifecycle`."""
        pass



# Copyright (C) 2011-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Testing app.inject functions."""

from __future__ import absolute_import, print_function, unicode_literals

__metaclass__ = type
__all__ = [
    ]


import unittest

from mailman.app.inject import inject_message, inject_text
from mailman.app.lifecycle import create_list
from mailman.email.message import Message
from mailman.testing.helpers import (
    get_queue_messages,
    specialized_message_from_string as message_from_string)
from mailman.testing.layers import ConfigLayer


NL = '\n'



class TestInjectMessage(unittest.TestCase):
    """Test message injection."""

    layer = ConfigLayer

    def setUp(self):
        self.mlist = create_list('test@example.com')
        self.msg = message_from_string("""\
From: anne@example.com
To: test@example.com
Subject: A test message
Message-ID: <first>
Date: Tue, 14 Jun 2011 21:12:00 -0400

Nothing.
""")
        # Python 2.7 has assertMultiLineEqual.  Let this work without bounds.
        self.maxDiff = None
        self.eq = getattr(self, 'assertMultiLineEqual', self.assertEqual)

    def test_inject_message(self):
        # Test basic inject_message() call.
        inject_message(self.mlist, self.msg)
        items = get_queue_messages('in')
        self.assertEqual(len(items), 1)
        self.eq(items[0].msg.as_string(), self.msg.as_string())
        self.assertEqual(items[0].msgdata['listname'], 'test@example.com')
        self.assertEqual(items[0].msgdata['original_size'],
                         len(self.msg.as_string()))

    def test_inject_message_with_recipients(self):
        # Explicit recipients end up in the metadata.
        recipients = ['bart@example.com', 'cris@example.com']
        inject_message(self.mlist, self.msg, recipients)
        items = get_queue_messages('in')
        self.assertEqual(items[0].msgdata['recipients'], recipients)

    def test_inject_message_to_queue(self):
        # Explicitly use a different queue.
        inject_message(self.mlist, self.msg, switchboard='virgin')
        items = get_queue_messages('in')
        self.assertEqual(len(items), 0)
        items = get_queue_messages('virgin')
        self.assertEqual(len(items), 1)
        self.eq(items[0].msg.as_string(), self.msg.as_string())
        self.assertEqual(items[0].msgdata['listname'], 'test@example.com')
        self.assertEqual(items[0].msgdata['original_size'],
                         len(self.msg.as_string()))

    def test_inject_message_without_message_id(self):
        # inject_message() adds a Message-ID header if it's missing.
        del self.msg['message-id']
        self.assertFalse('message-id' in self.msg)
        inject_message(self.mlist, self.msg)
        self.assertTrue('message-id' in self.msg)
        items = get_queue_messages('in')
        self.assertTrue('message-id' in items[0].msg)
        self.assertEqual(items[0].msg['message-id'], self.msg['message-id'])

    def test_inject_message_without_date(self):
        # inject_message() adds a Date header if it's missing.
        del self.msg['date']
        self.assertFalse('date' in self.msg)
        inject_message(self.mlist, self.msg)
        self.assertTrue('date' in self.msg)
        items = get_queue_messages('in')
        self.assertTrue('date' in items[0].msg)
        self.assertEqual(items[0].msg['date'], self.msg['date'])

    def test_inject_message_with_keywords(self):
        # Keyword arguments are copied into the metadata.
        inject_message(self.mlist, self.msg, foo='yes', bar='no')
        items = get_queue_messages('in')
        self.assertEqual(items[0].msgdata['foo'], 'yes')
        self.assertEqual(items[0].msgdata['bar'], 'no')

    def test_inject_message_id_hash(self):
        # When the injected message has a Message-ID header, the injected
        # message will also get an X-Message-ID-Hash header.
        inject_message(self.mlist, self.msg)
        items = get_queue_messages('in')
        self.assertEqual(items[0].msg['x-message-id-hash'],
                         '4CMWUN6BHVCMHMDAOSJZ2Q72G5M32MWB')

    def test_inject_message_id_hash_without_message_id(self):
        # When the injected message does not have a Message-ID header, a
        # Message-ID header will be added, and the injected message will also
        # get an X-Message-ID-Hash header.
        del self.msg['message-id']
        self.assertFalse('message-id' in self.msg)
        self.assertFalse('x-message-id-hash' in self.msg)
        inject_message(self.mlist, self.msg)
        items = get_queue_messages('in')
        self.assertTrue('message-id' in items[0].msg)
        self.assertTrue('x-message-id-hash' in items[0].msg)



class TestInjectText(unittest.TestCase):
    """Test text injection."""

    layer = ConfigLayer

    def setUp(self):
        self.mlist = create_list('test@example.com')
        self.text = b"""\
From: bart@example.com
To: test@example.com
Subject: A test message
Message-ID: <second>
Date: Tue, 14 Jun 2011 21:12:00 -0400

Nothing.
"""
        # Python 2.7 has a better equality tester for message texts.
        self.maxDiff = None
        self.eq = getattr(self, 'assertMultiLineEqual', self.assertEqual)

    def _remove_line(self, header):
        return NL.join(line for line in self.text.splitlines()
                       if not line.lower().startswith(header))

    def test_inject_text(self):
        # Test basic inject_text() call.
        inject_text(self.mlist, self.text)
        items = get_queue_messages('in')
        self.assertEqual(len(items), 1)
        self.assertTrue(isinstance(items[0].msg, Message))
        self.assertEqual(items[0].msg['x-message-id-hash'],
                         'GUXXQKNCHBFQAHGBFMGCME6HKZCUUH3K')
        # Delete that header because it is not in the original text.
        del items[0].msg['x-message-id-hash']
        self.eq(items[0].msg.as_string(), self.text)
        self.assertEqual(items[0].msgdata['listname'], 'test@example.com')
        self.assertEqual(items[0].msgdata['original_size'],
                         # Add back the X-Message-ID-Header which was in the
                         # message contributing to the original_size, but
                         # wasn't in the original text.  Don't forget the
                         # newline!
                         len(self.text) + 52)

    def test_inject_text_with_recipients(self):
        # Explicit recipients end up in the metadata.
        recipients = ['bart@example.com', 'cris@example.com']
        inject_text(self.mlist, self.text, recipients)
        items = get_queue_messages('in')
        self.assertEqual(items[0].msgdata['recipients'], recipients)

    def test_inject_text_to_queue(self):
        # Explicitly use a different queue.
        inject_text(self.mlist, self.text, switchboard='virgin')
        items = get_queue_messages('in')
        self.assertEqual(len(items), 0)
        items = get_queue_messages('virgin')
        self.assertEqual(len(items), 1)
        # Remove the X-Message-ID-Hash header which isn't in the original text.
        del items[0].msg['x-message-id-hash']
        self.eq(items[0].msg.as_string(), self.text)
        self.assertEqual(items[0].msgdata['listname'], 'test@example.com')
        self.assertEqual(items[0].msgdata['original_size'],
                         # Add back the X-Message-ID-Header which was in the
                         # message contributing to the original_size, but
                         # wasn't in the original text.  Don't forget the
                         # newline!
                         len(self.text) + 52)

    def test_inject_text_without_message_id(self):
        # inject_text() adds a Message-ID header if it's missing.
        filtered = self._remove_line('message-id')
        self.assertFalse('Message-ID' in filtered)
        inject_text(self.mlist, filtered)
        items = get_queue_messages('in')
        self.assertTrue('message-id' in items[0].msg)

    def test_inject_text_without_date(self):
        # inject_text() adds a Date header if it's missing.
        filtered = self._remove_line('date')
        self.assertFalse('date' in filtered)
        inject_text(self.mlist, self.text)
        items = get_queue_messages('in')
        self.assertTrue('date' in items[0].msg)

    def test_inject_text_adds_original_size(self):
        # The metadata gets an original_size attribute that is the length of
        # the injected text.
        inject_text(self.mlist, self.text)
        items = get_queue_messages('in')
        self.assertEqual(items[0].msgdata['original_size'],
                         # Add back the X-Message-ID-Header which was in the
                         # message contributing to the original_size, but
                         # wasn't in the original text.  Don't forget the
                         # newline!
                         len(self.text) + 52)

    def test_inject_text_with_keywords(self):
        # Keyword arguments are copied into the metadata.
        inject_text(self.mlist, self.text, foo='yes', bar='no')
        items = get_queue_messages('in')
        self.assertEqual(items[0].msgdata['foo'], 'yes')
        self.assertEqual(items[0].msgdata['bar'], 'no')

    def test_inject_message_id_hash(self):
        # When the injected message has a Message-ID header, the injected
        # message will also get an X-Message-ID-Hash header.
        inject_text(self.mlist, self.text)
        items = get_queue_messages('in')
        self.assertEqual(items[0].msg['x-message-id-hash'],
                         'GUXXQKNCHBFQAHGBFMGCME6HKZCUUH3K')

    def test_inject_message_id_hash_without_message_id(self):
        # When the injected message does not have a Message-ID header, a
        # Message-ID header will be added, and the injected message will also
        # get an X-Message-ID-Hash header.
        filtered = self._remove_line('message-id')
        self.assertFalse('Message-ID' in filtered)
        self.assertFalse('X-Message-ID-Hash' in filtered)
        inject_text(self.mlist, filtered)
        items = get_queue_messages('in')
        self.assertTrue('message-id' in items[0].msg)
        self.assertTrue('x-message-id-hash' in items[0].msg)



# Copyright (C) 2011-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Moderation tests."""

from __future__ import absolute_import, print_function, unicode_literals

__metaclass__ = type
__all__ = [
    'TestModeration',
    ]


import unittest

from mailman.app.lifecycle import create_list
from mailman.app.moderator import handle_message, hold_message
from mailman.interfaces.action import Action
from mailman.interfaces.requests import IListRequests
from mailman.runners.incoming import IncomingRunner
from mailman.runners.outgoing import OutgoingRunner
from mailman.runners.pipeline import PipelineRunner
from mailman.testing.helpers import (
    make_testable_runner, specialized_message_from_string)
from mailman.testing.layers import SMTPLayer



class TestModeration(unittest.TestCase):
    """Test moderation functionality."""

    layer = SMTPLayer

    def setUp(self):
        self._mlist = create_list('test@example.com')
        self._msg = specialized_message_from_string("""\
From: anne@example.com
To: test@example.com
Subject: hold me
Message-ID: <alpha>

""")
        self._in = make_testable_runner(IncomingRunner, 'in')
        self._pipeline = make_testable_runner(PipelineRunner, 'pipeline')
        self._out = make_testable_runner(OutgoingRunner, 'out')
        # Python 2.7 has assertMultiLineEqual.  Let this work without bounds.
        self.maxDiff = None
        self.eq = getattr(self, 'assertMultiLineEqual', self.assertEqual)

    def test_accepted_message_gets_posted(self):
        # A message that is accepted by the moderator should get posted to the
        # mailing list.  LP: #827697
        msgdata = dict(listname='test@example.com',
                       recipients=['bart@example.com'])
        request_id = hold_message(self._mlist, self._msg, msgdata)
        handle_message(self._mlist, request_id, Action.accept)
        self._in.run()
        self._pipeline.run()
        self._out.run()
        messages = list(SMTPLayer.smtpd.messages)
        self.assertEqual(len(messages), 1)
        message = messages[0]
        # We don't need to test the entire posted message, just the bits that
        # prove it got sent out.
        self.assertTrue('x-mailman-version' in message)
        self.assertTrue('x-peer' in message)
        # The X-Mailman-Approved-At header has local timezone information in
        # it, so test that separately.
        self.assertEqual(message['x-mailman-approved-at'][:-5],
                         'Mon, 01 Aug 2005 07:49:23 ')
        del message['x-mailman-approved-at']
        # The Message-ID matches the original.
        self.assertEqual(message['message-id'], '<alpha>')
        # Anne sent the message and the mailing list received it.
        self.assertEqual(message['from'], 'anne@example.com')
        self.assertEqual(message['to'], 'test@example.com')
        # The Subject header has the list's prefix.
        self.assertEqual(message['subject'], '[Test] hold me')
        # The list's -bounce address is the actual sender, and Bart is the
        # only actual recipient.  These headers are added by the testing
        # framework and don't show up in production.  They match the RFC 5321
        # envelope.
        self.assertEqual(message['x-mailfrom'], 'test-bounces@example.com')
        self.assertEqual(message['x-rcptto'], 'bart@example.com')

    def test_hold_action_alias_for_defer(self):
        # In handle_message(), the 'hold' action is the same as 'defer' for
        # purposes of this API.
        request_id = hold_message(self._mlist, self._msg)
        handle_message(self._mlist, request_id, Action.defer)
        # The message is still in the pending requests.
        requests_db = IListRequests(self._mlist)
        key, data = requests_db.get_request(request_id)
        self.assertEqual(key, '<alpha>')
        handle_message(self._mlist, request_id, Action.hold)
        key, data = requests_db.get_request(request_id)
        self.assertEqual(key, '<alpha>')



# Copyright (C) 2009-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Datetime utilities.

Use these functions to produce variable times rather than the built-in
datetime.datetime.now() and datetime.date.today().  These are better
instrumented for testing purposes.
"""


from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'DateFactory',
    'factory',
    'now',
    'today',
    ]


import datetime

from mailman.testing import layers



class DateFactory:
    """A factory for today() and now() that works with testing."""

    # The predictable time.
    predictable_now = None
    predictable_today = None

    def now(self, tz=None):
        # We can't automatically fast-forward because some tests require us to
        # stay on the same day for a while, e.g. autorespond.txt.
        return (self.predictable_now
                if layers.is_testing()
                else datetime.datetime.now(tz))

    def today(self):
        return (self.predictable_today
                if layers.is_testing()
                else datetime.date.today())

    @classmethod
    def reset(cls):
        cls.predictable_now = datetime.datetime(2005, 8, 1, 7, 49, 23)
        cls.predictable_today = cls.predictable_now.date()

    @classmethod
    def fast_forward(cls, days=1):
        cls.predictable_now += datetime.timedelta(days=days)
        cls.predictable_today = cls.predictable_now.date()


factory = DateFactory()
factory.reset()
today = factory.today
now = factory.now
layers.MockAndMonkeyLayer.register_reset(factory.reset)



# Copyright (C) 1998-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import time
import socket
import logging
import nntplib
import optparse
import email.Errors

from email.Parser import Parser
from flufl.lock import Lock, TimeOutError
from lazr.config import as_host_port

from mailman import MailList
from mailman import Message
from mailman import loginit
from mailman.configuration import config
from mailman.core.i18n import _
from mailman.core.switchboard import Switchboard
from mailman.version import MAILMAN_VERSION

# Work around known problems with some RedHat cron daemons
import signal
signal.signal(signal.SIGCHLD, signal.SIG_DFL)

NL = '\n'

log = None

class _ContinueLoop(Exception):
    pass



def parseargs():
    parser = optparse.OptionParser(version=MAILMAN_VERSION,
                                   usage=_("""\
%prog [options]

Poll the NNTP servers for messages to be gatewayed to mailing lists."""))
    parser.add_option('-C', '--config',
                      help=_('Alternative configuration file to use'))
    opts, args = parser.parse_args()
    if args:
        parser.print_help()
        print >> sys.stderr, _('Unexpected arguments')
        sys.exit(1)
    return opts, args, parser



_hostcache = {}

def open_newsgroup(mlist):
    # Split host:port if given.
    nntp_host, nntp_port = as_host_port(mlist.nntp_host, default_port=119)
    # Open up a "mode reader" connection to nntp server.  This will be shared
    # for all the gated lists having the same nntp_host.
    conn = _hostcache.get(mlist.nntp_host)
    if conn is None:
        try:
            conn = nntplib.NNTP(nntp_host, nntp_port,
                                readermode=True,
                                user=config.NNTP_USERNAME,
                                password=config.NNTP_PASSWORD)
        except (socket.error, nntplib.NNTPError, IOError), e:
            log.error('error opening connection to nntp_host: %s\n%s',
                      mlist.nntp_host, e)
            raise
        _hostcache[mlist.nntp_host] = conn
    # Get the GROUP information for the list, but we're only really interested
    # in the first article number and the last article number
    r, c, f, l, n = conn.group(mlist.linked_newsgroup)
    return conn, int(f), int(l)


def clearcache():
    for conn in set(_hostcache.values()):
        conn.quit()
    _hostcache.clear()



# This function requires the list to be locked.
def poll_newsgroup(mlist, conn, first, last, glock):
    listname = mlist.internal_name()
    # NEWNEWS is not portable and has synchronization issues.
    for num in range(first, last):
        glock.refresh()
        try:
            headers = conn.head(repr(num))[3]
            found_to = False
            beenthere = False
            for header in headers:
                i = header.find(':')
                value = header[:i].lower()
                if i > 0 and value == 'to':
                    found_to = True
                # FIXME 2010-02-16 barry use List-Post header.
                if value <> 'x-beenthere':
                    continue
                if header[i:] == ': %s' % mlist.posting_address:
                    beenthere = True
                    break
            if not beenthere:
                body = conn.body(repr(num))[3]
                # Usenet originated messages will not have a Unix envelope
                # (i.e. "From " header).  This breaks Pipermail archiving, so
                # we will synthesize one.  Be sure to use the format searched
                # for by mailbox.UnixMailbox._isrealfromline().  BAW: We use
                # the -bounces address here in case any downstream clients use
                # the envelope sender for bounces; I'm not sure about this,
                # but it's the closest to the old semantics.
                lines = ['From %s  %s' % (mlist.GetBouncesEmail(),
                                          time.ctime(time.time()))]
                lines.extend(headers)
                lines.append('')
                lines.extend(body)
                lines.append('')
                p = Parser(Message.Message)
                try:
                    msg = p.parsestr(NL.join(lines))
                except email.Errors.MessageError, e:
                    log.error('email package exception for %s:%d\n%s',
                              mlist.linked_newsgroup, num, e)
                    raise _ContinueLoop
                if found_to:
                    del msg['X-Originally-To']
                    msg['X-Originally-To'] = msg['To']
                    del msg['To']
                msg['To'] = mlist.posting_address
                # Post the message to the locked list
                inq = Switchboard(config.INQUEUE_DIR)
                inq.enqueue(msg,
                            listname=mlist.internal_name(),
                            fromusenet=True)
                log.info('posted to list %s: %7d', listname, num)
        except nntplib.NNTPError, e:
            log.exception('NNTP error for list %s: %7d', listname, num)
        except _ContinueLoop:
            continue
        # Even if we don't post the message because it was seen on the
        # list already, update the watermark
        mlist.usenet_watermark = num



def process_lists(glock):
    for listname in config.list_manager.names:
        glock.refresh()
        # Open the list unlocked just to check to see if it is gating news to
        # mail.  If not, we're done with the list.  Otherwise, lock the list
        # and gate the group.
        mlist = MailList.MailList(listname, lock=False)
        if not mlist.gateway_to_mail:
            continue
        # Get the list's watermark, i.e. the last article number that we gated
        # from news to mail.  None means that this list has never polled its
        # newsgroup and that we should do a catch up.
        watermark = getattr(mlist, 'usenet_watermark', None)
        # Open the newsgroup, but let most exceptions percolate up.
        try:
            conn, first, last = open_newsgroup(mlist)
        except (socket.error, nntplib.NNTPError):
            break
        log.info('%s: [%d..%d]', listname, first, last)
        try:
            try:
                if watermark is None:
                    mlist.Lock(timeout=config.LIST_LOCK_TIMEOUT)
                    # This is the first time we've tried to gate this
                    # newsgroup.  We essentially do a mass catch-up, otherwise
                    # we'd flood the mailing list.
                    mlist.usenet_watermark = last
                    log.info('%s caught up to article %d', listname, last)
                else:
                    # The list has been polled previously, so now we simply
                    # grab all the messages on the newsgroup that have not
                    # been seen by the mailing list.  The first such article
                    # is the maximum of the lowest article available in the
                    # newsgroup and the watermark.  It's possible that some
                    # articles have been expired since the last time gate_news
                    # has run.  Not much we can do about that.
                    start = max(watermark + 1, first)
                    if start > last:
                        log.info('nothing new for list %s', listname)
                    else:
                        mlist.Lock(timeout=config.LIST_LOCK_TIMEOUT)
                        log.info('gating %s articles [%d..%d]',
                                 listname, start, last)
                        # Use last+1 because poll_newsgroup() employes a for
                        # loop over range, and this will not include the last
                        # element in the list.
                        poll_newsgroup(mlist, conn, start, last + 1, glock)
            except TimeOutError:
                log.error('Could not acquire list lock: %s', listname)
        finally:
            if mlist.Locked():
                mlist.Save()
                mlist.Unlock()
        log.info('%s watermark: %d', listname, mlist.usenet_watermark)



def main():
    opts, args, parser = parseargs()
    config.load(opts.config)

    GATENEWS_LOCK_FILE = os.path.join(config.LOCK_DIR, 'gate_news.lock')
    LOCK_LIFETIME = config.hours(2)

    loginit.initialize(propagate=True)
    log = logging.getLogger('mailman.fromusenet')

    try:
        with Lock(GATENEWS_LOCK_FILE,
                  # It's okay to hijack this
                  lifetime=LOCK_LIFETIME) as lock:
            process_lists(lock)
        clearcache()
    except TimeOutError:
        log.error('Could not acquire gate_news lock')



if __name__ == '__main__':
    main()



# Copyright (C) 2002-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""The email commands 'join' and 'subscribe'."""

from __future__ import absolute_import, print_function, unicode_literals

__metaclass__ = type
__all__ = [
    'Join',
    'Subscribe',
    'Leave',
    'Unsubscribe',
    ]


from email.utils import formataddr, parseaddr
from zope.component import getUtility
from zope.interface import implements

from mailman.core.i18n import _
from mailman.interfaces.command import ContinueProcessing, IEmailCommand
from mailman.interfaces.member import DeliveryMode, MemberRole
from mailman.interfaces.registrar import IRegistrar
from mailman.interfaces.subscriptions import ISubscriptionService
from mailman.interfaces.usermanager import IUserManager



class Join:
    """The email 'join' command."""

    implements(IEmailCommand)

    name = 'join'
    # XXX 2012-02-29 BAW: DeliveryMode.summary is not yet supported.
    argument_description = '[digest=<no|mime|plain>]'
    description = _("""\
You will be asked to confirm your subscription request and you may be issued a
provisional password.

By using the 'digest' option, you can specify whether you want digest delivery
or not.  If not specified, the mailing list's default delivery mode will be
used.
""")
    short_description = _('Join this mailing list.')

    def process(self, mlist, msg, msgdata, arguments, results):
        """See `IEmailCommand`."""
        # Parse the arguments.
        delivery_mode = self._parse_arguments(arguments, results)
        if delivery_mode is ContinueProcessing.no:
            return ContinueProcessing.no
        display_name, address = parseaddr(msg['from'])
        # Address could be None or the empty string.
        if not address:
            address = msg.sender
        if not address:
            print(_('$self.name: No valid address found to subscribe'),
                  file=results)
            return ContinueProcessing.no
        # Have we already seen one join request from this user during the
        # processing of this email?
        joins = getattr(results, 'joins', set())
        if address in joins:
            # Do not register this join.
            return ContinueProcessing.yes
        joins.add(address)
        results.joins = joins
        person = formataddr((display_name, address))
        # Is this person already a member of the list?  Search for all
        # matching memberships.
        members = getUtility(ISubscriptionService).find_members(
            address, mlist.fqdn_listname, MemberRole.member)
        if len(members) > 0:
            print(_('$person is already a member'), file=results)
        else:
            getUtility(IRegistrar).register(mlist, address, 
                                            display_name, delivery_mode)
            print(_('Confirmation email sent to $person'), file=results)
        return ContinueProcessing.yes

    def _parse_arguments(self, arguments, results):
        """Parse command arguments.

        :param arguments: The sequences of arguments as given to the
            `process()` method.
        :param results: The results object.
        :return: The delivery mode, None, or ContinueProcessing.no on error.
        """
        mode = DeliveryMode.regular
        for argument in arguments:
            parts = argument.split('=', 1)
            if len(parts) != 2 or parts[0] != 'digest':
                print(self.name, _('bad argument: $argument'),
                      file=results)
                return ContinueProcessing.no
            mode = {
                'no': DeliveryMode.regular,
                'plain': DeliveryMode.plaintext_digests,
                'mime': DeliveryMode.mime_digests,
                }.get(parts[1])
            if mode is None:
                print(self.name, _('bad argument: $argument'),
                      file=results)
                return ContinueProcessing.no
        return mode



class Subscribe(Join):
    """The email 'subscribe' command (an alias for 'join')."""

    name = 'subscribe'
    description = _("An alias for 'join'.")
    short_description = description



class Leave:
    """The email 'leave' command."""

    implements(IEmailCommand)

    name = 'leave'
    argument_description = ''
    description = _("""Leave this mailing list.  

You may be asked to confirm your request.""")
    short_description = _('Leave this mailing list.')

    def process(self, mlist, msg, msgdata, arguments, results):
        """See `IEmailCommand`."""
        email = msg.sender
        if not email:
            print(_('$self.name: No valid email address found to unsubscribe'),
                  file=results)
            return ContinueProcessing.no
        user_manager = getUtility(IUserManager)
        user = user_manager.get_user(email)
        if user is None:
            print(_('No registered user for email address: $email'),
                  file=results)
            return ContinueProcessing.no
        # The address that the -leave command was sent from, must be verified.
        # Otherwise you could link a bogus address to anyone's account, and
        # then send a leave command from that address.
        if user_manager.get_address(email).verified_on is None:
            print(_('Invalid or unverified email address: $email'),
                  file=results)
            return ContinueProcessing.no
        for user_address in user.addresses:
            # Only recognize verified addresses.
            if user_address.verified_on is None:
                continue
            member = mlist.members.get_member(user_address.email)
            if member is not None:
                break
        else:
            # None of the user's addresses are subscribed to this mailing list.
            print(_(
                '$self.name: $email is not a member of $mlist.fqdn_listname'),
                file=results)
            return ContinueProcessing.no
        member.unsubscribe()
        person = formataddr((user.display_name, email))
        print(_('$person left $mlist.fqdn_listname'), file=results)
        return ContinueProcessing.yes


class Unsubscribe(Leave):
    """The email 'unsubscribe' command (an alias for 'leave')."""

    name = 'unsubscribe'
    description = _("An alias for 'leave'.")
    short_description = description



# Copyright (C) 2000-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Archive runner."""

from __future__ import absolute_import, print_function, unicode_literals

__metaclass__ = type
__all__ = [
    'ArchiveRunner',
    ]


import os
import logging

from datetime import datetime
from email.utils import parsedate_tz, mktime_tz, formatdate
from flufl.lock import Lock
from lazr.config import as_timedelta

from mailman.config import config
from mailman.core.runner import Runner

log = logging.getLogger('mailman.error')



class ArchiveRunner(Runner):
    """The archive runner."""

    def _dispose(self, mlist, msg, msgdata):
        # Support clobber_date, i.e. setting the date in the archive to the
        # received date, not the (potentially bogus) Date: header of the
        # original message.
        clobber = False
        original_date = msg.get('date')
        received_time = formatdate(msgdata['received_time'])
        # FIXME 2012-03-23 BAW: LP: #963612
        ## if not original_date:
        ##     clobber = True
        ## elif int(config.archiver.pipermail.clobber_date_policy) == 1:
        ##     clobber = True
        ## elif int(config.archiver.pipermail.clobber_date_policy) == 2:
        ##     # What's the timestamp on the original message?
        ##     timetup = parsedate_tz(original_date)
        ##     now = datetime.now()
        ##     try:
        ##         if not timetup:
        ##             clobber = True
        ##         else:
        ##             utc_timestamp = datetime.fromtimestamp(mktime_tz(timetup))
        ##             date_skew = as_timedelta(
        ##                 config.archiver.pipermail.allowable_sane_date_skew)
        ##             clobber = (abs(now - utc_timestamp) > date_skew)
        ##     except (ValueError, OverflowError):
        ##         # The likely cause of this is that the year in the Date: field
        ##         # is horribly incorrect, e.g. (from SF bug # 571634):
        ##         # Date: Tue, 18 Jun 0102 05:12:09 +0500
        ##         # Obviously clobber such dates.
        ##         clobber = True
        ## if clobber:
        ##     del msg['date']
        ##     del msg['x-original-date']
        ##     msg['Date'] = received_time
        ##     if original_date:
        ##         msg['X-Original-Date'] = original_date
        # Always put an indication of when we received the message.
        msg['X-List-Received-Date'] = received_time
        # While a list archiving lock is acquired, archive the message.
        with Lock(os.path.join(mlist.data_path, 'archive.lck')):
            for archiver in config.archivers:
                # A problem in one archiver should not prevent other archivers
                # from running.
                try:
                    archiver.archive_message(mlist, msg)
                except Exception:
                    log.exception('Broken archiver: %s' % archiver.name)



# Copyright (C) 2011-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""REST root object tests."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    ]


import unittest

from urllib2 import HTTPError

from mailman.testing.helpers import call_api
from mailman.testing.layers import RESTLayer



class TestSystem(unittest.TestCase):
    layer = RESTLayer

    def test_system_url_too_long(self):
        # /system/foo/bar is not allowed.
        try:
            # For Python 2.6.
            call_api('http://localhost:9001/3.0/system/foo/bar')
        except HTTPError as exc:
            self.assertEqual(exc.code, 400)
        else:
            raise AssertionError('Expected HTTPError')

    def test_system_url_not_preferences(self):
        # /system/foo where `foo` is not `preferences`.
        try:
            # For Python 2.6.
            call_api('http://localhost:9001/3.0/system/foo')
        except HTTPError as exc:
            self.assertEqual(exc.code, 400)
        else:
            raise AssertionError('Expected HTTPError')

    def test_system_preferences_are_read_only(self):
        # /system/preferences are read-only.
        try:
            # For Python 2.6.
            call_api('http://localhost:9001/3.0/system/preferences', {
                     'acknowledge_posts': True,
                     }, method='PATCH')
        except HTTPError as exc:
            self.assertEqual(exc.code, 405)
        else:
            raise AssertionError('Expected HTTPError')
        # /system/preferences are read-only.
        try:
            # For Python 2.6.
            call_api('http://localhost:9001/3.0/system/preferences', {
                'acknowledge_posts': False,
                'delivery_mode': 'regular',
                'delivery_status': 'enabled',
                'hide_address': True,
                'preferred_language': 'en',
                'receive_list_copy': True,
                'receive_own_postings': True,
                }, method='PUT')
        except HTTPError as exc:
            self.assertEqual(exc.code, 405)
        else:
            raise AssertionError('Expected HTTPError')



# Copyright (C) 2008-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Interfaces defining email commands."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'ContinueProcessing',
    'ICLISubCommand',
    'IEmailCommand',
    'IEmailResults',
    ]


from flufl.enum import Enum
from zope.interface import Interface, Attribute



class ContinueProcessing(Enum):
    """Should `IEmailCommand.process()` continue or not."""
    no = 0
    yes = 1



class IEmailResults(Interface):
    """The email command results object."""

    output = Attribute('An output file object for printing results to.')



class IEmailCommand(Interface):
    """An email command."""

    name = Attribute('Command name as seen in a -request email.')

    argument_description = Attribute('Description of command arguments.')

    description = Attribute('Command help.')

    def process(mlist, msg, msgdata, arguments, results):
        """Process the email command.

        :param mlist: The mailing list target of the command.
        :param msg: The original message object.
        :param msgdata: The message metadata.
        :param arguments: The command arguments tuple.
        :param results: An IEmailResults object for these commands.
        :return: A `ContinueProcessing` enum specifying whether to continue
            processing or not.
        """



class ICLISubCommand(Interface):
    """A command line interface subcommand."""

    name = Attribute('The command name; must be unique')

    __doc__ = Attribute('The command short help')

    def add(parser, command_parser):
        """Add the subcommand to the subparser.

        :param parser: The argument parser.
        :type parser: `argparse.ArgumentParser`
        :param subparser: The command subparser.
        :type subparser: `argparse.ArgumentParser`
        """

    def process(args):
        """Process the subcommand.

        :param args: The namespace, as passed in by argparse.
        :type args: `argparse.Namespace`
        """



# Copyright (C) 1998-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import absolute_import, unicode_literals

import os
import sys
import errno
import shutil
import optparse

from flufl.lock import Lock

from mailman.Archiver.HyperArch import HyperArchive
from mailman.Defaults import hours
from mailman.configuration import config
from mailman.core.i18n import _
from mailman.initialize import initialize
from mailman.version import MAILMAN_VERSION



def parseargs():
    parser = optparse.OptionParser(version=MAILMAN_VERSION,
                                   usage=_("""\
%%prog [options] listname [mbox]

Rebuild a list's archive.

Use this command to rebuild the archives for a mailing list.  You may want to
do this if you edit some messages in an archive, or remove some messages from
an archive.

Where 'mbox' is the path to a list's complete mbox archive.  Usually this will
be some path in the archives/private directory.  For example:

% bin/arch mylist archives/private/mylist.mbox/mylist.mbox

'mbox' is optional.  If it is missing, it is calculated from the listname.
"""))
    parser.add_option('-q', '--quiet',
                      dest='verbose', default=True, action='store_false',
                      help=_('Make the archiver output less verbose'))
    parser.add_option('--wipe',
                      default=False, action='store_true',
                      help=_("""\
First wipe out the original archive before regenerating.  You usually want to
specify this argument unless you're generating the archive in chunks."""))
    parser.add_option('-s', '--start',
                      default=None, type='int', metavar='N',
                      help=_("""\
Start indexing at article N, where article 0 is the first in the mbox.
Defaults to 0."""))
    parser.add_option('-e', '--end',
                      default=None, type='int', metavar='M',
                      help=_("""\
End indexing at article M.  This script is not very efficient with respect to
memory management, and for large archives, it may not be possible to index the
mbox entirely.  For that reason, you can specify the start and end article
numbers."""))
    parser.add_option('-C', '--config',
                      help=_('Alternative configuration file to use'))
    opts, args = parser.parse_args()
    if len(args) < 1:
        parser.print_help()
        print >> sys.stderr, _('listname is required')
        sys.exit(1)
    if len(args) > 2:
        parser.print_help()
        print >> sys.stderr, _('Unexpected arguments')
        sys.exit(1)
    return parser, opts, args



def main():
    parser, opts, args = parseargs()
    initialize(opts.config)

    listname = args[0].lower().strip()
    if len(args) < 2:
        mbox = None
    else:
        mbox = args[1]

    # Open the mailing list object
    mlist = config.list_manager.get(listname)
    if mlist is None:
        parser.error(_('No such list: $listname'))
    if mbox is None:
        mbox = mlist.ArchiveFileName()

    # Set the default language to the list's preferred language.
    _.default = mlist.preferred_language
    # Lay claim to the archive's lock file.  This is so no other post can
    # mess up the archive while we're processing it.  Try to pick a
    # suitably long period of time for the lock lifetime even though we
    # really don't know how long it will take.
    #
    # XXX processUnixMailbox() should refresh the lock.
    lock_path = os.path.join(mlist.data_path, '.archiver.lck')
    with Lock(lock_path, lifetime=int(hours(3))):
        # Try to open mbox before wiping old archive.
        try:
            fp = open(mbox)
        except IOError, e:
            if e.errno == errno.ENOENT:
                print >> sys.stderr, _('Cannot open mbox file: $mbox')
            else:
                print >> sys.stderr, e
            sys.exit(1)
        # Maybe wipe the old archives
        if opts.wipe:
            if mlist.scrub_nondigest:
                # TK: save the attachments dir because they are not in mbox
                saved = False
                atchdir = os.path.join(mlist.archive_dir(), 'attachments')
                savedir = os.path.join(mlist.archive_dir() + '.mbox',
                                       'attachments')
                try:
                    os.rename(atchdir, savedir)
                    saved = True
                except OSError, e:
                    if e.errno != errno.ENOENT:
                        raise
            shutil.rmtree(mlist.archive_dir())
            if mlist.scrub_nondigest and saved:
                os.renames(savedir, atchdir)

        archiver = HyperArchive(mlist)
        archiver.VERBOSE = opts.verbose
        try:
            archiver.processUnixMailbox(fp, opts.start, opts.end)
        finally:
            archiver.close()
        fp.close()



# Copyright (C) 2009-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Module stuff."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'makedirs',
    'umask',
    ]


import os
import errno



class umask:
    """Manage the umask for the with statement."""

    def __init__(self, umask):
        self._umask = umask
        self._old_umask = None

    def __enter__(self):
        assert self._old_umask is None, 'Unexpected existing umask'
        self._old_umask = os.umask(self._umask)

    def __exit__(self, *exc_info):
        assert self._old_umask is not None, 'No previous umask'
        os.umask(self._old_umask)
        # Do not suppress exceptions.
        return False



def makedirs(path, mode=02775):
    """Create a directory hierarchy, ensuring permissions.

    Other than just calling os.makedirs(), this ensures that the umask is
    reset so that the makedirs mode will be honored.

    :param path: The directory path to create.
    :type path: string
    :param mode: The numeric permission mode to use.
    :type mode: int
    """
    try:
        with umask(0):
            os.makedirs(path, mode)
    except OSError as error:
        # Ignore the exceptions if the directory already exists.
        if error.errno != errno.EEXIST:
            raise
    # Some systems such as FreeBSD ignore mkdir's mode, so walk the just
    # created directories and try to set the mode, ignoring any OSErrors that
    # occur here.
    for dirpath, dirnames, filenames in os.walk(path):
        try:
            os.chmod(dirpath, mode)
        except OSError:
            pass



# Copyright (C) 2010-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for config.pck imports."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    ]


import cPickle
import unittest

from mailman.app.lifecycle import create_list, remove_list
from mailman.testing.layers import ConfigLayer
from mailman.utilities.importer import import_config_pck
from pkg_resources import resource_filename



class TestBasicImport(unittest.TestCase):
    layer = ConfigLayer

    def setUp(self):
        self._mlist = create_list('blank@example.com')
        pickle_file = resource_filename('mailman.testing', 'config.pck')
        with open(pickle_file) as fp:
            self._pckdict = cPickle.load(fp)

    def tearDown(self):
        remove_list(self._mlist.fqdn_listname, self._mlist)

    def _import(self):
        import_config_pck(self._mlist, self._pckdict)

    def test_display_name(self):
        # The mlist.display_name gets set from the old list's real_name.
        self.assertEqual(self._mlist.display_name, 'Blank')
        self._import()
        self.assertEqual(self._mlist.display_name, 'Test')

    def test_mail_host(self):
        # The mlist.mail_host gets set.
        self.assertEqual(self._mlist.mail_host, 'example.com')
        self._import()
        self.assertEqual(self._mlist.mail_host, 'heresy.example.org')

    def test_rfc2369_headers(self):
        self._mlist.include_list_post_header = False
        self._mlist.include_rfc2369_headers = False
        self._import()
        self.assertTrue(self._mlist.include_list_post_header)
        self.assertTrue(self._mlist.include_rfc2369_headers)



# Copyright (C) 2010-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""REST for mailing lists."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'AList',
    'AllLists',
    'ListConfiguration',
    'ListsForDomain',
    ]


from operator import attrgetter
from restish import http, resource
from zope.component import getUtility

from mailman.app.lifecycle import create_list, remove_list
from mailman.interfaces.domain import BadDomainSpecificationError
from mailman.interfaces.listmanager import (
    IListManager, ListAlreadyExistsError)
from mailman.interfaces.member import MemberRole
from mailman.interfaces.subscriptions import ISubscriptionService
from mailman.rest.configuration import ListConfiguration
from mailman.rest.helpers import (
    CollectionMixin, etag, no_content, path_to, restish_matcher)
from mailman.rest.members import AMember, MemberCollection
from mailman.rest.moderation import HeldMessages
from mailman.rest.validator import Validator



@restish_matcher
def member_matcher(request, segments):
    """A matcher of member URLs inside mailing lists.

    e.g. /<role>/aperson@example.org
    """
    if len(segments) != 2:
        return None
    try:
        role = MemberRole[segments[0]]
    except ValueError:
        # Not a valid role.
        return None
    # No more segments.
    # XXX 2010-02-25 barry Matchers are undocumented in restish; they return a
    # 3-tuple of (match_args, match_kws, segments).
    return (), dict(role=role, email=segments[1]), ()


@restish_matcher
def roster_matcher(request, segments):
    """A matcher of all members URLs inside mailing lists.

    e.g. /roster/<role>
    """
    if len(segments) != 2 or segments[0] != 'roster':
        return None
    try:
        return (), dict(role=MemberRole[segments[1]]), ()
    except ValueError:
        # Not a valid role.
        return None


@restish_matcher
def config_matcher(request, segments):
    """A matcher for a mailing list's configuration resource.

    e.g. /config
    e.g. /config/description
    """
    if len(segments) < 1 or segments[0] != 'config':
        return None
    if len(segments) == 1:
        return (), {}, ()
    if len(segments) == 2:
        return (), dict(attribute=segments[1]), ()
    # More segments are not allowed.
    return None



class _ListBase(resource.Resource, CollectionMixin):
    """Shared base class for mailing list representations."""

    def _resource_as_dict(self, mlist):
        """See `CollectionMixin`."""
        return dict(
            display_name=mlist.display_name,
            fqdn_listname=mlist.fqdn_listname,
            list_name=mlist.list_name,
            mail_host=mlist.mail_host,
            member_count=mlist.members.member_count,
            volume=mlist.volume,
            self_link=path_to('lists/{0}'.format(mlist.fqdn_listname)),
            )

    def _get_collection(self, request):
        """See `CollectionMixin`."""
        return list(getUtility(IListManager))


class AList(_ListBase):
    """A mailing list."""

    def __init__(self, list_name):
        self._mlist = getUtility(IListManager).get(list_name)

    @resource.GET()
    def mailing_list(self, request):
        """Return a single mailing list end-point."""
        if self._mlist is None:
            return http.not_found()
        return http.ok([], self._resource_as_json(self._mlist))

    @resource.DELETE()
    def delete_list(self, request):
        """Delete the named mailing list."""
        if self._mlist is None:
            return http.not_found()
        remove_list(self._mlist.fqdn_listname, self._mlist)
        return no_content()

    @resource.child(member_matcher)
    def member(self, request, segments, role, email):
        """Return a single member representation."""
        if self._mlist is None:
            return http.not_found()
        members = getUtility(ISubscriptionService).find_members(
            email, self._mlist.fqdn_listname, role)
        if len(members) == 0:
            return http.not_found()
        assert len(members) == 1, 'Too many matches'
        return AMember(members[0].member_id)

    @resource.child(roster_matcher)
    def roster(self, request, segments, role):
        """Return the collection of all a mailing list's members."""
        if self._mlist is None:
            return http.not_found()
        return MembersOfList(self._mlist, role)

    @resource.child(config_matcher)
    def config(self, request, segments, attribute=None):
        """Return a mailing list configuration object."""
        if self._mlist is None:
            return http.not_found()
        return ListConfiguration(self._mlist, attribute)

    @resource.child()
    def held(self, request, segments):
        """Return a list of held messages for the mailign list."""
        if self._mlist is None:
            return http.not_found()
        return HeldMessages(self._mlist)



class AllLists(_ListBase):
    """The mailing lists."""

    @resource.POST()
    def create(self, request):
        """Create a new mailing list."""
        try:
            validator = Validator(fqdn_listname=unicode)
            mlist = create_list(**validator(request))
        except ListAlreadyExistsError:
            return http.bad_request([], b'Mailing list exists')
        except BadDomainSpecificationError as error:
            return http.bad_request([], b'Domain does not exist {0}'.format(
                error.domain))
        except ValueError as error:
            return http.bad_request([], str(error))
        # wsgiref wants headers to be bytes, not unicodes.
        location = path_to('lists/{0}'.format(mlist.fqdn_listname))
        # Include no extra headers or body.
        return http.created(location, [], None)

    @resource.GET()
    def collection(self, request):
        """/lists"""
        resource = self._make_collection(request)
        return http.ok([], etag(resource))



class MembersOfList(MemberCollection):
    """The members of a mailing list."""

    def __init__(self, mailing_list, role):
        super(MembersOfList, self).__init__()
        self._mlist = mailing_list
        self._role = role

    def _get_collection(self, request):
        """See `CollectionMixin`."""
        # Overrides _MemberBase._get_collection() because we only want to
        # return the members from the requested roster.
        roster = self._mlist.get_roster(self._role)
        address_of_member = attrgetter('address.email')
        return list(sorted(roster.members, key=address_of_member))


class ListsForDomain(_ListBase):
    """The mailing lists for a particular domain."""

    def __init__(self, domain):
        self._domain = domain

    @resource.GET()
    def collection(self, request):
        """/domains/<domain>/lists"""
        resource = self._make_collection(request)
        return http.ok([], etag(resource))

    def _get_collection(self, request):
        """See `CollectionMixin`."""
        return list(self._domain.mailing_lists)



# Copyright (C) 2009-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""The 'mailman' command dispatcher."""

from __future__ import absolute_import, print_function, unicode_literals

__metaclass__ = type
__all__ = [
    'main',
    ]


import os
import argparse

from zope.interface.verify import verifyObject

from mailman.app.finder import find_components
from mailman.core.i18n import _
from mailman.core.initialize import initialize
from mailman.interfaces.command import ICLISubCommand
from mailman.version import MAILMAN_VERSION_FULL



def main():
    """bin/mailman"""
    # Create the basic parser and add all globally common options.
    parser = argparse.ArgumentParser(
        description=_("""\
        The GNU Mailman mailing list management system
        Copyright 1998-2012 by the Free Software Foundation, Inc.
        http://www.list.org
        """),
        formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument(
        '-v', '--version',
        action='version', version=MAILMAN_VERSION_FULL,
        help=_('Print this version string and exit'))
    parser.add_argument(
        '-C', '--config',
        help=_("""\
        Configuration file to use.  If not given, the environment variable
        MAILMAN_CONFIG_FILE is consulted and used if set.  If neither are
        given, a default configuration file is loaded."""))
    # Look at all modules in the mailman.bin package and if they are prepared
    # to add a subcommand, let them do so.  I'm still undecided as to whether
    # this should be pluggable or not.  If so, then we'll probably have to
    # partially parse the arguments now, then initialize the system, then find
    # the plugins.  Punt on this for now.
    subparser = parser.add_subparsers(title='Commands')
    subcommands = []
    for command_class in find_components('mailman.commands', ICLISubCommand):
        command = command_class()
        verifyObject(ICLISubCommand, command)
        subcommands.append(command)
    # --help should display the subcommands by alphabetical order, except that
    # 'mailman help' should be first.
    def sort_function(command, other):
        """Sorting helper."""
        if command.name == 'help':
            return -1
        elif other.name == 'help':
            return 1
        else:
            return cmp(command.name, other.name)
    subcommands.sort(cmp=sort_function)
    for command in subcommands:
        command_parser = subparser.add_parser(
            command.name, help=_(command.__doc__))
        command.add(parser, command_parser)
        command_parser.set_defaults(func=command.process)
    args = parser.parse_args()
    if len(args.__dict__) == 0:
        # No arguments or subcommands were given.
        parser.print_help()
        parser.exit()
    # Before actually performing the subcommand, we need to initialize the
    # Mailman system, and in particular, we must read the configuration file.
    config_file = os.getenv('MAILMAN_CONFIG_FILE')
    if config_file is None:
        if args.config is not None:
            config_file = os.path.abspath(os.path.expanduser(args.config))

    initialize(config_file)
    # Perform the subcommand option.
    args.func(args)



# Copyright (C) 2007-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Application of list styles to new and existing lists."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'DefaultStyle',
    ]


# XXX Styles need to be reconciled with lazr.config.

import datetime

from zope.interface import implements

from mailman.core.i18n import _
from mailman.interfaces.action import Action, FilterAction
from mailman.interfaces.bounce import UnrecognizedBounceDisposition
from mailman.interfaces.digests import DigestFrequency
from mailman.interfaces.autorespond import ResponseAction
from mailman.interfaces.mailinglist import Personalization, ReplyToMunging
from mailman.interfaces.nntp import NewsModeration
from mailman.interfaces.styles import IStyle



class DefaultStyle:
    """The default (i.e. legacy) style."""

    implements(IStyle)

    name = 'default'
    priority = 0    # the lowest priority style

    def apply(self, mailing_list):
        """See `IStyle`."""
        # For cut-n-paste convenience.
        mlist = mailing_list
        # List identity.
        mlist.display_name = mlist.list_name.capitalize()
        mlist.list_id = '{0.list_name}.{0.mail_host}'.format(mlist)
        mlist.include_rfc2369_headers = True
        mlist.include_list_post_header = True
        # Most of these were ripped from the old MailList.InitVars() method.
        mlist.volume = 1
        mlist.post_id = 1
        mlist.new_member_options = 256
        mlist.respond_to_post_requests = True
        mlist.advertised = True
        mlist.max_num_recipients = 10
        mlist.max_message_size = 40 # KB
        mlist.reply_goes_to_list = ReplyToMunging.no_munging
        mlist.reply_to_address = ''
        mlist.first_strip_reply_to = False
        mlist.admin_immed_notify = True
        mlist.admin_notify_mchanges = False
        mlist.require_explicit_destination = True
        mlist.send_reminders = True
        mlist.send_welcome_message = True
        mlist.send_goodbye_message = True
        mlist.bounce_matching_headers = """
# Lines that *start* with a '#' are comments.
to: friend@public.com
message-id: relay.comanche.denmark.eu
from: list@listme.com
from: .*@uplinkpro.com
"""
        mlist.header_matches = []
        mlist.anonymous_list = False
        mlist.description = ''
        mlist.info = ''
        mlist.welcome_message_uri = 'mailman:///welcome.txt'
        mlist.goodbye_message_uri = ''
        mlist.subscribe_policy = 1
        mlist.subscribe_auto_approval = []
        mlist.unsubscribe_policy = 0
        mlist.private_roster = 1
        mlist.obscure_addresses = True
        mlist.admin_member_chunksize = 30
        mlist.administrivia = True
        mlist.preferred_language = 'en'
        mlist.collapse_alternatives = True
        mlist.convert_html_to_plaintext = False
        mlist.filter_action = FilterAction.discard
        mlist.filter_content = False
        # Digest related variables
        mlist.digestable = True
        mlist.digest_is_default = False
        mlist.mime_is_default_digest = False
        mlist.digest_size_threshold = 30 # KB
        mlist.digest_send_periodic = True
        mlist.digest_header_uri = None
        mlist.digest_footer_uri = (
            'mailman:///$listname/$language/footer-generic.txt')
        mlist.digest_volume_frequency = DigestFrequency.monthly
        mlist.next_digest_number = 1
        mlist.nondigestable = True
        mlist.personalize = Personalization.none
        # New sender-centric moderation (privacy) options
        mlist.default_member_action = Action.defer
        mlist.default_nonmember_action = Action.hold
        # Archiver
        mlist.archive = True
        mlist.archive_private = 0
        mlist.archive_volume_frequency = 1
        mlist.emergency = False
        mlist.member_moderation_notice = ''
        mlist.accept_these_nonmembers = []
        mlist.hold_these_nonmembers = []
        mlist.reject_these_nonmembers = []
        mlist.discard_these_nonmembers = []
        mlist.forward_auto_discards = True
        mlist.generic_nonmember_action = 1
        mlist.nonmember_rejection_notice = ''
        # Max autoresponses per day.  A mapping between addresses and a
        # 2-tuple of the date of the last autoresponse and the number of
        # autoresponses sent on that date.
        mlist.subject_prefix = _('[$mlist.display_name] ')
        mlist.header_uri = None
        mlist.footer_uri = 'mailman:///$listname/$language/footer-generic.txt'
        # Set this to Never if the list's preferred language uses us-ascii,
        # otherwise set it to As Needed.
        if mlist.preferred_language.charset == 'us-ascii':
            mlist.encode_ascii_prefixes = 0
        else:
            mlist.encode_ascii_prefixes = 2
        # scrub regular delivery
        mlist.scrub_nondigest = False
        # automatic discarding
        mlist.max_days_to_hold = 0
        # Autoresponder
        mlist.autorespond_owner = ResponseAction.none
        mlist.autoresponse_owner_text = ''
        mlist.autorespond_postings = ResponseAction.none
        mlist.autoresponse_postings_text = ''
        mlist.autorespond_requests = ResponseAction.none
        mlist.autoresponse_request_text = ''
        mlist.autoresponse_grace_period = datetime.timedelta(days=90)
        # Bounces
        mlist.forward_unrecognized_bounces_to = (
            UnrecognizedBounceDisposition.administrators)
        mlist.process_bounces = True
        mlist.bounce_score_threshold = 5.0
        mlist.bounce_info_stale_after = datetime.timedelta(days=7)
        mlist.bounce_you_are_disabled_warnings = 3
        mlist.bounce_you_are_disabled_warnings_interval = (
            datetime.timedelta(days=7))
        mlist.bounce_notify_owner_on_disable = True
        mlist.bounce_notify_owner_on_removal = True
        # This holds legacy member related information.  It's keyed by the
        # member address, and the value is an object containing the bounce
        # score, the date of the last received bounce, and a count of the
        # notifications left to send.
        mlist.bounce_info = {}
        # New style delivery status
        mlist.delivery_status = {}
        # NNTP gateway
        mlist.nntp_host = ''
        mlist.linked_newsgroup = ''
        mlist.gateway_to_news = False
        mlist.gateway_to_mail = False
        mlist.news_prefix_subject_too = True
        # In patch #401270, this was called newsgroup_is_moderated, but the
        # semantics weren't quite the same.
        mlist.news_moderation = NewsModeration.none
        # Topics
        #
        # `topics' is a list of 4-tuples of the following form:
        #
        #     (name, pattern, description, emptyflag)
        #
        # name is a required arbitrary string displayed to the user when they
        # get to select their topics of interest
        #
        # pattern is a required verbose regular expression pattern which is
        # used as IGNORECASE.
        #
        # description is an optional description of what this topic is
        # supposed to match
        #
        # emptyflag is a boolean used internally in the admin interface to
        # signal whether a topic entry is new or not (new ones which do not
        # have a name or pattern are not saved when the submit button is
        # pressed).
        mlist.topics = []
        mlist.topics_enabled = False
        mlist.topics_bodylines_limit = 5
        # This is a mapping between user "names" (i.e. addresses) and
        # information about which topics that user is interested in.  The
        # values are a list of topic names that the user is interested in,
        # which should match the topic names in mlist.topics above.
        #
        # If the user has not selected any topics of interest, then the rule
        # is that they will get all messages, and they will not have an entry
        # in this dictionary.
        mlist.topics_userinterest = {}
        # The processing chain that messages posted to this mailing list get
        # processed by.
        mlist.posting_chain = 'default-posting-chain'
        # The default pipeline to send accepted messages through to the
        # mailing list's members.
        mlist.posting_pipeline = 'default-posting-pipeline'
        # The processing chain that messages posted to this mailing list's
        # -owner address gets processed by.
        mlist.owner_chain = 'default-owner-chain'
        # The default pipeline to send -owner email through.
        mlist.owner_pipeline = 'default-owner-pipeline'

    def match(self, mailing_list, styles):
        """See `IStyle`."""
        # If no other styles have matched, then the default style matches.
        if len(styles) == 0:
            styles.append(self)



# Copyright (C) 2007-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Interface representing domains."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'BadDomainSpecificationError',
    'DomainCreatedEvent',
    'DomainCreatingEvent',
    'DomainDeletedEvent',
    'DomainDeletingEvent',
    'IDomain',
    'IDomainManager',
    ]


from zope.interface import Interface, Attribute

from mailman.core.errors import MailmanError



class BadDomainSpecificationError(MailmanError):
    """The specification of a virtual domain is invalid or duplicated."""

    def __init__(self, domain):
        super(BadDomainSpecificationError, self).__init__(domain)
        self.domain = domain


class DomainCreatingEvent:
    """A domain is about to be created."""

    def __init__(self, mail_host):
        self.mail_host = mail_host


class DomainCreatedEvent:
    """A domain was created."""

    def __init__(self, domain):
        self.domain = domain


class DomainDeletingEvent:
    """A domain is about to be deleted."""

    def __init__(self, domain):
        self.domain = domain


class DomainDeletedEvent:
    """A domain was deleted."""

    def __init__(self, mail_host):
        self.mail_host = mail_host



class IDomain(Interface):
    """Interface representing domains."""

    mail_host = Attribute('The host name for email for this domain.')

    url_host = Attribute(
        'The host name for the web interface for this domain.')

    base_url = Attribute("""\
    The base url for the Mailman server at this domain, which includes the
    scheme and host name.""")

    scheme = Attribute(
        """The protocol scheme used to contact this list's server.""")

    description = Attribute(
        'The human readable description of the domain name.')

    contact_address = Attribute("""\
    The contact address for the human at this domain.
    E.g. postmaster@example.com""")

    mailing_lists = Attribute(
        'All mailing lists for this domain.')

    def confirm_url(token=''):
        """The url used for various forms of confirmation.

        :param token: The confirmation token to use in the url.
        :type token: string
        :return: The confirmation url.
        :rtype: string
        """



class IDomainManager(Interface):
    """The manager of domains."""

    def add(mail_host, description=None, base_url=None, contact_address=None):
        """Add a new domain.

        :param mail_host: The email host name for the domain.
        :type mail_host: string
        :param description: The description of the domain.
        :type description: string
        :param base_url: The base url, including the scheme for the web
            interface of the domain.  If not given, it defaults to
            http://`mail_host`/
        :type base_url: string
        :param contact_address: The email contact address for the human
            managing the domain.  If not given, defaults to
            postmaster@`mail_host`
        :type contact_address: string
        :return: The new domain object
        :rtype: `IDomain`
        :raises `BadDomainSpecificationError`: when the `mail_host` is
            already registered.
        """

    def remove(mail_host):
        """Remove the domain.

        :param mail_host: The email host name of the domain to remove.
        :type mail_host: string
        :raises KeyError: if the named domain does not exist.
        """

    def __getitem__(mail_host):
        """Return the named domain.

        :param mail_host: The email host name of the domain to remove.
        :type mail_host: string
        :return: The domain object.
        :rtype: `IDomain`
        :raises KeyError: if the named domain does not exist.
        """

    def get(mail_host, default=None):
        """Return the named domain.

        :param mail_host: The email host name of the domain to remove.
        :type mail_host: string
        :param default: What to return if the named domain does not exist.
        :type default: object
        :return: The domain object or None if the named domain does not exist.
        :rtype: `IDomain`
        """

    def __iter__():
        """An iterator over all the domains.

        :return: iterator over `IDomain`.
        """

    def __contains__(mail_host):
        """Is this a known domain?

        :param mail_host: An email host name.
        :type mail_host: string
        :return: True if this domain is known.
        :rtype: bool
        """



# Copyright (C) 2009-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""The 'help' subcommand."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'Help',
    ]


from zope.interface import implements

from mailman.interfaces.command import ICLISubCommand



class Help:
    # Lowercase, to match argparse's default --help text.
    """show this help message and exit"""

    implements(ICLISubCommand)

    name = 'help'

    def add(self, parser, command_parser):
        """See `ICLISubCommand`."""
        self.parser = parser

    def process(self, args):
        """See `ICLISubCommand`."""
        self.parser.print_help()



# Copyright (C) 2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Template downloader with cache."""

from __future__ import absolute_import, print_function, unicode_literals

__metaclass__ = type
__all__ = [
    'ITemplateLoader',
    ]


from zope.interface import Interface



class ITemplateLoader(Interface):
    """The template downloader utility."""

    def get(uri):
        """Download the named URI, and return the response and content.

        This API uses `urllib2`_ so consult its documentation for details.

        .. _`urllib2`: http://docs.python.org/library/urllib2.html

        :param uri: The URI of the resource.  These may be any URI supported
            by `urllib2` and also `mailman:` URIs for internal resources.
        :type uri: string
        :return: An open file object as defined by urllib2.
        """
